@@ -202,7 +202,7 @@ rigidly along with this one."
     (concat result "\\)[ \t\n>]")))
 
 (defconst mhtml-complex-tags
-  '( "when" "while" "defsubst" "defun" "defmacro" "defstruct"
+  '( "when" "while" "with" "defsubst" "defun" "defmacro" "defstruct"
      "define-tag" "define-function"
      "with-open-database" "with-open-stream" "server-push"
      "comment" "verbatim" "small-caps" "plain-text" "ptext" "section"
@@ -469,10 +469,10 @@ of the start of the containing expression."
 (put 'if 'mhtml-indent-function 2)
 (put 'ifeq 'mhtml-indent-function 3)
 (put 'var-case 'mhtml-indent-function 'defun)
-;;; (put 'set-var 'mhtml-indent-function 'defun)
-;;; (put 'get-var 'mhtml-indent-function 'defun)
-;;; (put 'set-session-var 'mhtml-indent-function 'defun)
-;;; (put 'get-session-var 'mhtml-indent-function 'defun)
+(put 'set-var 'mhtml-indent-function 'defun)
+(put 'get-var 'mhtml-indent-function 'defun)
+(put 'set-session-var 'mhtml-indent-function 'defun)
+(put 'get-session-var 'mhtml-indent-function 'defun)
 (put 'PROG 'mhtml-indent-function 'defun)
 (put 'DATABASE-QUERY 'mhtml-indent-function 'defun)
 (put 'DATABASE-LOAD-RECORD 'mhtml-indent-function 'defun)
@@ -487,10 +487,10 @@ of the start of the containing expression."
 (put 'IF 'mhtml-indent-function 2)
 (put 'IFEQ 'mhtml-indent-function 3)
 (put 'VAR-CASE 'mhtml-indent-function 'defun)
-;;; (put 'SET-VAR 'mhtml-indent-function 'defun)
-;;; (put 'GET-VAR 'mhtml-indent-function 'defun)
-;;; (put 'SET-SESSION-VAR 'mhtml-indent-function 'defun)
-;;; (put 'GET-SESSION-VAR 'mhtml-indent-function 'defun)
+(put 'SET-VAR 'mhtml-indent-function 'defun)
+(put 'GET-VAR 'mhtml-indent-function 'defun)
+(put 'SET-SESSION-VAR 'mhtml-indent-function 'defun)
+(put 'GET-SESSION-VAR 'mhtml-indent-function 'defun)
 
 (defun mhtml-indent-sexp (&optional endpos)
   "Indent each line of the list starting just after point.
