@@ -774,7 +774,10 @@ process_request (void)
     }
 
   if (req != (HTTP_REQUEST *)NULL)
-    result = http_handle_request (req, connection);
+    {
+      result = http_handle_request (req, connection);
+      mhttpd_free_request (req);
+    }
 
   if (result && result->page && result->page->buffer)
     {
@@ -795,4 +798,12 @@ process_request (void)
       fwrite (result->page->buffer + i, result->page->bindex, 1, stdout);
       fflush (stdout);
     }
+  if (result)
+    {
+      if (result->spec)
+	mhttpd_free_doc_spec (result->spec);
+
+      page_free_page (result->page);
+      free (result);
+    }
 }
