@@ -39,7 +39,8 @@ static void pf_foreach (PFunArgs);
 static void pf_array_concat (PFunArgs);
 static void pf_sort (PFunArgs);
 
-static PFunDesc functab[] = {
+static PFunDesc func_table[] =
+{
   /*   tag	     complex? debug_level	   code    */
   { "ARRAY-SIZE",	0,	0,		pf_array_size },
   { "ARRAY-MEMBER",	0,	0,		pf_array_member },
@@ -52,22 +53,7 @@ static PFunDesc functab[] = {
   { (char *)NULL,	0,	0,		(PFunHandler *)NULL }
 };
 
-PACKAGE_INITIALIZER(initialize_array_funcs)
-
-void
-initialize_array_funcs (Package *pack)
-{
-  register int i;
-  Symbol *sym;
-
-  /* Install the names and pointers. */
-  for (i = 0; functab[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (mhtml_function_package, functab[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&functab[i]);
-    }
-}
+PACKAGE_INITIALIZER (initialize_array_functions)
 
 /* <array-size foo> --> number of allocated elements in FOO. */
 static void
@@ -296,7 +282,8 @@ pf_foreach (PFunArgs)
     {
       Symbol *array = symbol_lookup (array_var);
 
-      if ((array != (Symbol *)NULL) && (array->type == symtype_STRING))
+      if ((array != (Symbol *)NULL) && (array->type == symtype_STRING) &&
+	  (array->values_index > 0) && (array->values != (char **)NULL))
 	{
 	  register int i;
 
@@ -334,6 +321,11 @@ pf_foreach (PFunArgs)
 		step = -1;
 	    }
 
+	  /* Final sanity check.  Make sure that START and END are within
+	     the bounds of the final array. */
+	  if ((step < 0) && start >= array->values_index)
+	    start = array->values_index;
+
 	  if (((step < 0) && (end_index < start_index)) ||
 	      ((step > 0) && (end_index > start_index)))
 	    {
@@ -344,12 +336,14 @@ pf_foreach (PFunArgs)
 		{
 		  Symbol *element = symbol_remove (element_var);
 		  PAGE *code;
+		  int line = parser_current_lineno;
 
 		  symbol_free (element);
 		  element = symbol_intern (element_var);
 		  symbol_add_value (element, array->values[i]);
 		  code = page_copy_page (body);
 		  page_process_page_internal (code);
+		  parser_current_lineno = line;
 
 		  if (code != (PAGE *)NULL)
 		    {
