@@ -41,28 +41,17 @@
 #  endif
 #endif /* NEW_CGI_EXEC */
 
-PACKAGE_INITIALIZER (initialize_cgi_exec_functions)
 
 static void pf_cgi_exec (PFunArgs);
 
-static PFunDesc cgi_exec_table[] = {
+static PFunDesc func_table[] =
+{
   { "CGI-EXEC",		0, 0, pf_cgi_exec },
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
-void
-initialize_cgi_exec_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; cgi_exec_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, cgi_exec_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&cgi_exec_table[i]);
-    }
-}
+PACKAGE_INITIALIZER (initialize_cgi_exec_functions)
+DOC_SECTION (FILE-OPERATORS)
 
 #if defined (NEW_CGI_EXEC)
 static char **
@@ -364,8 +353,19 @@ shell_execute (char *command, char **output_textp, char **errors_textp,
 }
 #endif /* !NEW_CGI_EXEC */
 
-static void
-pf_cgi_exec (PFunArgs)
+DEFUN (pf_cgi_exec,
+       pathname &optional arg... &key output=varname errors=varname
+       timeout=seconds nowait=true,
+"Execute the system function named by <var pathname>, perhaps passing
+it arguments <var arg ... argn>.
+
+If <var output=varname> is supplied, then the variable referenced by
+<var varname> receives the output of the command.  Otherwise, the
+resultant output is placed in the page.
+
+If <var errors=varname> is supplied, then <var varname> receives the
+error output of the command. Otherwise, the resultant output is placed
+in <funref language-operators SYSTEM-ERROR-OUTPUT>.")
 {
   if (vars != (Package *)NULL)
     {
