@@ -45,7 +45,8 @@ static void pf_database_first_match (PFunArgs);
 static void pf_database_next_match (PFunArgs);
 #endif /* NOT_NEEDED_WITH_DATABASE_QUERY */
 
-PFunDesc dbfunc_table[] = {
+static PFunDesc func_table[] =
+{
   { "WITH-OPEN-DATABASE",	1, 0, pf_with_open_database },
   { "DATABASE-LOAD-RECORD",	0, 0, pf_database_load_record },
   { "DATABASE-DELETE-RECORD",	0, 0, pf_database_delete_record },
@@ -63,19 +64,7 @@ PFunDesc dbfunc_table[] = {
 };
 
 PACKAGE_INITIALIZER (initialize_db_functions)
-void
-initialize_db_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; dbfunc_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, dbfunc_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&dbfunc_table[i]);
-    }
-}
+DOC_SECTION (DATABASE-COMMANDS)
 
 static long
 random_number (void)
@@ -328,8 +317,24 @@ dbkey_function_args (Package *vars, DBFILE *db, DBOBJ **key, int key_required)
 static char *varname = (char *)NULL;
 static int varname_size = 0;
 
-static void
-pf_database_load_record (PFunArgs)
+DEFUN (pf_database_load_record, dbvar key &key package=package-name,
+"Load the variables from the record specified by <var key> in the
+database referenced by <var dbvar>.  If <var package=package-name> is
+given, the record variables are stored into the specified package,
+instead of the current package.
+
+Upon success, this function returns the string <code>\"true\"</code>.
+
+<example>
+<with-open-database db /tmp/file.db mode=read>
+  <set-var loaded? = <database-load-record db <get-var name> package=foo>>
+</with-open-database>
+
+<when <get-var loaded?>>
+  The record was loaded successfully, and the
+  value of NAME is <get-var foo::name>.
+</when>
+</example>")
 {
   char *prefix;
   int prefix_len;
@@ -1112,7 +1117,6 @@ pf_database_query (PFunArgs)
       register int i;
       DBOBJ *key = database_firstkey (db);
       DBOBJ *nextkey;
-      Package *save = CurrentPackage; 
       DBRecord **records = (DBRecord **)NULL;
       int rec_slots = 0;
       int rec_index = 0;
@@ -1140,9 +1144,9 @@ pf_database_query (PFunArgs)
 	    {
 	      char *expr_result;
 
-	      symbol_set_default_package (db_fields);
+	      symbol_push_package (db_fields);
 	      expr_result = mhtml_evaluate_string (expr);
-	      symbol_set_default_package (save);
+	      symbol_pop_package ();
 
 	      /* If satisfied, save this key. */
 	      if (!empty_string_p (expr_result))
@@ -1272,9 +1276,9 @@ pf_database_query (PFunArgs)
 		    ((format_limit < 0) || (i < format_limit)));
 		   i++)
 		{
-		  symbol_set_default_package (records[i]->contents);
+		  symbol_push_package (records[i]->contents);
 		  temp = mhtml_evaluate_string (format_expr);
-		  symbol_set_default_package (save);
+		  symbol_pop_package ();
 
 		  if (temp)
 		    {
