@@ -33,31 +33,22 @@
 static void pf_get_file_properties (PFunArgs);
 static void pf_set_file_properties (PFunArgs);
 static void pf_directory_contents (PFunArgs);
+static void pf_file_exists (PFunArgs);
 
 static char *unix_username_from_uid (uid_t uid);
 static void unix_add_date (Symbol *sym, long ticks);
 
-PFunDesc filefunc_table[] = {
+static PFunDesc func_table[] =
+{
   { "GET-FILE-PROPERTIES",	0, 0, pf_get_file_properties },
   { "SET-FILE-PROPERTIES",	0, 0, pf_set_file_properties },
   { "DIRECTORY-CONTENTS",	0, 0, pf_directory_contents },
+  { "FILE-EXISTS",		0, 0, pf_file_exists },
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
 PACKAGE_INITIALIZER (initialize_file_functions)
-void
-initialize_file_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; filefunc_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, filefunc_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&filefunc_table[i]);
-    }
-}
+DOC_SECTION (FILE-OPERATORS)
 
 /* <get-file-properties /www/nirvana/docs/welcome.mhtml>
 
@@ -66,9 +57,9 @@ initialize_file_functions (Package *package)
         (full-name . "/www/nirvana/docs/welcome.mhtml")
 	(size . 2188)
 	(type . FILE)
-	(created 1 21 95)
-	(written 1 21 95)
-	(read 3 21 96)
+	(created 1 21 95 01 01 00 298398348)
+	(written 1 21 95 01 01 00 298349349)
+	(read    3 21 96 01 01 00 928948938)
 	(creator . bfox))
 
      Values for TYPE: FILE, DIRECTORY, EXECUTABLE. */
@@ -149,7 +140,7 @@ static void
 unix_add_date (Symbol *sym, long ticks)
 {
   struct tm *ti = localtime ((const time_t *)&ticks);
-  char buffer[20];
+  char buffer[40];
 
   sprintf (buffer, "%02d", ti->tm_mon + 1);
   symbol_add_value (sym, buffer);
@@ -168,6 +159,9 @@ unix_add_date (Symbol *sym, long ticks)
 
   sprintf (buffer, "%02d", ti->tm_sec);
   symbol_add_value (sym, buffer);
+
+  sprintf (buffer, "%ld", (unsigned long) ticks);
+  symbol_add_value (sym, buffer);
 }
 
 static char *
@@ -194,8 +188,98 @@ unix_username_from_uid (uid_t uid)
    information about the named path.  Pathnames must be given fully; they
    are *not* relative to web space in any way.  This is probably a huge
    security hole, but it can't be any worse than CGI-EXEC.  Yeeesh. */
-static void
-pf_get_file_properties (PFunArgs)
+DEFUN (pf_get_file_properties, pathname,
+"Return an association-list containing operating system information
+about the file or directory named by <var path>. <var pathname> must be
+given fully; it is <i>not</i> relative to Web space in any way.
+
+If the file exists and is accessible, the members of the returned
+association-list which are guaranteed to be present are:
+
+<table border>
+  <tr>
+    <td align=left><b>Member</b></td>
+    <td align=left><b>Value</b></td>
+    <td align=left><b>Description</b></td>
+  </tr>
+  <tr>
+     <td align=left> <b>NAME</b></td>
+     <td align=left><i>welcome.mhtml</i></td>
+     <td align=left>
+        The name of the file or directory, without any of the path
+        information.
+     </td>
+  </tr>
+  <tr>
+     <td align=left><b>FULL-NAME</b></td>
+     <td align=left>/www/site/docs/welcome.mhtml</td>
+     <td align=left>
+       The name of the file or directory, with full path
+       information.  This should be identical to <var PATH> as
+       received by <b><code>get-file-properties</code></b>.
+     </td>
+   </tr>
+   <tr>
+     <td align=left> <b>SIZE</b> </td>
+     <td align=left>2188</td>
+     <td align=left>
+       The size of the file in bytes.
+     </td>
+   </tr>
+   <tr>
+     <td align=left><b>TYPE</b></td>
+     <td align=left>FILE</td>
+     <td align=left>
+        The <i>type</i> of the file.  This will either be
+       <b><code>FILE</code></b> or <b><code>DIRECTORY</code></b>.
+     </td>
+   </tr>
+   <tr>
+     <td align=center colspan=3>
+       <b>In addition to the above fields, the following fields appear
+       on Unix based systems.</b>
+     </td>
+   </tr>
+   <tr>
+     <td align=left><b>CREATED</b></td>
+     <td align=left>6 29 96 10 3 24</td>
+     <td align=left>
+       The date on which this file was created.  The value is an
+       array, with ordered values being: month, day, year, hours,
+       minutes, and seconds.
+     </td>
+   </tr>
+   <tr>
+     <td align=left><b>WRITTEN</b></td>
+     <td align=left>6 29 96 10 3 24</td>
+     <td align=left>
+       The date on which this file was last written.  The value is an
+       array, with ordered values being: month, day, year, hours,
+       minutes, and seconds.
+     </td>
+    </tr>
+    <tr>
+      <td align=left><b>READ</b></td>
+      <td align=left>6 30 96 19 27 51</td>
+      <td align=left>
+        The date on which this file was last read.  The value is an
+        array, with ordered values being: month, day, year, hours,
+        minutes, and seconds.
+      </td>
+    </tr>
+    <tr>
+       <td align=left><b>CREATOR</b></td>
+       <td align=left>bfox</td>
+       <td align=left>
+         The system identifier of the user who created this file.
+       </td>
+     </tr>
+</table>
+
+Example:
+<complete-example>
+<get-file-properties />
+<complete-example>")
 {
   char *filename = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *result = get_file_properties (filename);
@@ -211,15 +295,49 @@ pf_get_file_properties (PFunArgs)
     free (filename);
 }
 
+DEFUN (pf_file_exists, pathname,
+"Returns \"true\" if <var pathname> is an existing file or directory.
+
+<var pathname> is absolute -- it is looked up in absolute file system
+space, not in Web space.  To find out if a file <code>foo</code> in
+the current directory exists, use:
+<example>
+<file-exists <thisdir>/foo>
+</example>")
+{
+  char *filename = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (filename))
+    {
+      struct stat finfo;
+
+      if ((stat (filename, &finfo)) != -1)
+	{
+	  bprintf_insert (page, start, "true");
+	  *newstart += 4;
+	  if (debug_level > 5)
+	    page_debug ("<file-exists %s> --> true", filename);
+	}
+      else if (debug_level > 5)
+	page_debug ("<file-exists %s> -->", filename);
+    }
+}
+
 /* Not yet implemented. */
 static void
 pf_set_file_properties (PFunArgs)
 {
 }
 
-/* Return a directory as an alist of alists. */
-static void
-pf_directory_contents (PFunArgs)
+DEFUN (pf_directory_contents,
+       pathname &optional package-name &key matching=pattern,
+"Returns a newline separated list of association lists for
+the files matching <var pattern>.
+
+When <var package-name> is supplied, each variable in
+<var package-name> is the name of a file in <var pathname>, and
+the value of each variable is the association list for that
+file.")
 {
   char *dirname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *matching = mhtml_evaluate_string (get_value (vars, "matching"));
