@@ -89,12 +89,12 @@ forms_input_data (int argc, char *argv[])
       symbol = symbol_intern ("PROGRAM-ARGUMENTS");
       symbol->values = (char **)xmalloc (argc * sizeof (char *));
       symbol->values_slots = argc;
-      symbol->values_index = argc;
+      symbol->values_index = 0;
 
       for (i = 1; i < argc; i++)
-	symbol->values[i - 1] = strdup (argv[i]);
+	symbol->values[symbol->values_index++] = strdup (argv[i]);
 
-      symbol->values[i - 1] = (char *)NULL;
+      symbol->values[symbol->values_index] = (char *)NULL;
     }
 
   /* Now get all of the environment variables. */
@@ -260,6 +260,11 @@ forms_parse_data_string (const char *input, Package *package)
   free (buffer);
 }
 
+static char hexchars[]="0123456789ABCDEF";
+
+#define hi_nibble(c) hexchars[(((c & 0x000000FF) & 0xF0) >> 4)]
+#define lo_nibble(c) hexchars[(((c & 0x000000FF) & 0X0F) >> 0)]
+
 /* Turn SYMBOLS into a string suitable for appending onto a URL.
    This means that we encode special characters, and write name
    value pairs into a new string.
@@ -290,7 +295,7 @@ forms_unparse_items (Symbol **symbols)
 	  pair = (char *)xmalloc
 	    (1 + (3 * (name_len + strlen (symbol->values[j]))));
 
-	  for (from = 0, to = 0; (c = name[from]) != '\0'; from++)
+	  for (from = 0, to = 0; (c = (name[from])) != '\0'; from++)
 	    {
 	      if ((isalnum (c)) || (strchr (".-_@:", c) != (char *)NULL))
 		pair[to++] = c;
@@ -298,8 +303,9 @@ forms_unparse_items (Symbol **symbols)
 		pair[to++] = '+';
 	      else
 		{
-		  sprintf (pair + to, "%%%02X", c);
-		  to += 3;
+		  pair[to++] = '%';
+		  pair[to++] = hi_nibble (c);
+		  pair[to++] = lo_nibble (c);
 		}
 	    }
 
@@ -313,8 +319,9 @@ forms_unparse_items (Symbol **symbols)
 		pair[to++] = '+';
 	      else
 		{
-		  sprintf (pair + to, "%%%02X", c);
-		  to += 3;
+		  pair[to++] = '%';
+		  pair[to++] = hi_nibble (c);
+		  pair[to++] = lo_nibble (c);
 		}
 	    }
 
