@@ -84,6 +84,8 @@ static void pf_sql_transact (PFunArgs);
 /* Reporting of db errors to Meta-HTML environment  */
 #define GSQL_DEFAULT_ERRMSG NULL
 
+static int database_environment_level = 0;
+
 /* *************************************************************** */
 /*								   */
 /*	      Growable stacks for databases and cursors		   */
@@ -358,8 +360,6 @@ get_cursor_ref (Package *vars)
 			 && (! (gsql_field_type (ftype) == GSQL_REAL)) \
 			 && (! (gsql_field_type (ftype) == GSQL_DOUBLE)))
 
-static int database_environment_level = 0;
-
 /* We pass an string which contains a numeric representation of the
    SQL escape code character, such as "39" */
 static void
@@ -561,6 +561,8 @@ pf_with_open_database (PFunArgs)
   VOLATILE char *nolock   = mhtml_evaluate_string (get_value (vars, "NOLOCK"));
   int jump_again = 0;
 
+  if (!open_databases) initialize_database_stack ();
+
   /* No errors yet! */
   gsql_clear_error_message ();
 
@@ -658,9 +660,7 @@ pf_with_open_database (PFunArgs)
 /* <database-exec-sql dbvar query-string>
    Executes an SQL query, within the scope of with-open-database.
 
-   returns TRUE if no error (GSQL_ERROR), NULL otherwise.
-*/
-
+   returns TRUE if no error (GSQL_ERROR), NULL otherwise. */
 static void
 pf_database_exec_sql (PFunArgs)
 {
@@ -1110,7 +1110,6 @@ pf_database_next_record (PFunArgs)
 		xfree (t);
 	      }
 
-
 	      /* If the user does not want variables to have their
 		 appended values persist across calls to next-record,
 		 then we loop here to null out all the symbols which
@@ -1263,7 +1262,6 @@ pf_database_set_options (PFunArgs)
 
       if (symbol_lookup_in_package (vars, "SQL-PREFIX-TABLENAMES"))
 	gsql_database_set_prefix_tablenames (db, prefix_tablenames);
-
     }
 
   xfree (escape_code);
@@ -1563,7 +1561,6 @@ pf_database_query (PFunArgs)
       if (query_result != (gsql_result *)NULL)
 	{
 	  char *expr = get_positional_arg (vars, 1);
-	  Package *save = CurrentPackage;
 	  DBRecord **records = (DBRecord **)NULL;
 	  int window_start = 0;
 	  int window_length = 9999999;
@@ -1629,9 +1626,9 @@ pf_database_query (PFunArgs)
 		     the context of this record. */
 		  if (expr != (char *)NULL)
 		    {
-		      symbol_set_default_package (db_fields);
+		      symbol_push_package (db_fields);
 		      expr_result = mhtml_evaluate_string (expr);
-		      symbol_set_default_package (save);
+		      symbol_pop_package ();
 		    }
 
 		  /* If satisfied, save this record. */
@@ -1669,7 +1666,7 @@ pf_database_query (PFunArgs)
 
 	  /* Count the rest of the records? */
 	  {
-	    char *countvar = mhtml_evaluate_string (get_value (vars, "count"));
+	    char *countvar = mhtml_evaluate_string (get_value (vars, "COUNT"));
 
 	    if (!empty_string_p (countvar))
 	      {
@@ -1723,9 +1720,9 @@ pf_database_query (PFunArgs)
 			       ((format_limit < 0) || (i < format_limit)));
 		       i++)
 		    {
-		      symbol_set_default_package (records[i]->contents);
+		      symbol_push_package (records[i]->contents);
 		      temp = mhtml_evaluate_string (format_expr);
-		      symbol_set_default_package (save);
+		      symbol_pop_package ();
 
 		      if (temp)
 			{
@@ -1868,7 +1865,24 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
 static gsql_field *
 lookup_fieldname (char *name, gsql_result *result)
 {
-  int i, num_fields;
+  register int i, num_fields;
+
+#if 0
+  static char namebuff[1024];
+  /* We can't do this, because alists which are returned from the ...info
+     commands would have different names than those which we allowed in the
+     storage commands.  It is too confusing for the user, so we just don't
+     do it. */
+
+  /* Pleasant on the eye.  Allow dashes in the field name. */
+  {
+    char *temp;
+
+    strcpy (namebuff, name);
+    while ((temp = strchr (namebuff, '-')) != (char *)NULL) *temp = '_';
+    name = namebuff;
+  }
+#endif
 
   num_fields = gsql_num_fields (result);
 
@@ -1877,11 +1891,11 @@ lookup_fieldname (char *name, gsql_result *result)
       gsql_field *field;
 
       field = gsql_fetch_field (result, i);
-      if (!strcasecmp (name, gsql_field_name (field)))
-	return field;
+      if (strcasecmp (name, gsql_field_name (field)) == 0)
+	return (field);
     }
 
-  return (gsql_field *)NULL;
+  return ((gsql_field *)NULL);
 }
 
 /* <database-save-record db key var1 var2 ... varN
@@ -2042,7 +2056,7 @@ pf_database_save_record (PFunArgs)
 
 	  /* Execute the query. */
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
 	  status = gsql_query (db, query->buffer, 0);
 	  bprintf_free_buffer (query);
@@ -2116,7 +2130,7 @@ pf_database_save_record (PFunArgs)
 
 	  /* Execute the query. */
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
 	  status = gsql_query (db, query->buffer, 1);
 	  bprintf_free_buffer (query);
@@ -2207,7 +2221,7 @@ pf_database_load_record (PFunArgs)
 	  QUOTE_KEYVAL_IF_NEEDED;
 
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
 	  status = gsql_query (db, query->buffer, 0);
 	  bprintf_free_buffer (query);
@@ -2335,7 +2349,7 @@ pf_database_delete_record (PFunArgs)
 	  QUOTE_KEYVAL_IF_NEEDED;
 
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
 	  status = gsql_query (db, query->buffer, 0);
 	  bprintf_free_buffer (query);
@@ -2388,6 +2402,18 @@ pf_database_save_package (PFunArgs)
   /* No errors yet! */
   gsql_clear_error_message ();
 
+  if (db != (Database *)NULL)
+    {
+      if (empty_string_p (key))
+	gsql_save_error_message (db, "Missing KEY argument");
+
+      if (empty_string_p (table))
+	gsql_save_error_message (db, "Missing TABLE=tablename argument");
+
+      if (empty_string_p (keyname))
+	gsql_save_error_message (db, "Missing keyname=field argument");
+    }
+
   if ((db != (Database *)NULL) &&
       (db->connected == 1) &&
       (database_environment_level != 0) &&
@@ -2425,14 +2451,18 @@ pf_database_save_package (PFunArgs)
 	char *packname = mhtml_evaluate_string (get_positional_arg (vars, 2));
 
 	if (!empty_string_p (packname))
-	  symbols = symbol_package_symbols (packname);
+	  {
+	    symbols = symbol_package_symbols (packname);
+	  }
+	else
+	  {
+	    gsql_save_error_message (db, "missing package-name");
+	    errors_found++;
+	  }
 
 	xfree (packname);
       }
 
-      if (symbols == (Symbol **)NULL)
-	errors_found++;
-
       if  (!errors_found &&
 	   (empty_string_p (method) || (strcasecmp (method, "INSERT") == 0)))
 	{
@@ -2443,8 +2473,7 @@ pf_database_save_package (PFunArgs)
 	     ('keyval', 'val', 'val', ...)" */
 	  bprintf (query, "INSERT INTO %s (", table);
 
-	  /* Comma-separated list of all the variable names,
-	     including key. */
+	  /* Comma-separated list of all the variable names, including key. */
 	  bprintf (query, "%s", gsql_field_name (keyfield));
 
 	  for (i = 0; (sym = symbols[i]) != (Symbol *)NULL; i++)
@@ -2488,7 +2517,7 @@ pf_database_save_package (PFunArgs)
 		  bprintf (query, ",");
 
 		  /* We only use single quotes around CHAR type data. */
-		  if (CHARTYPE(field))
+		  if (CHARTYPE (field))
 		    bprintf (query, "'");
 
 		  /* Print out the various values.  If there are none,
@@ -2513,7 +2542,7 @@ pf_database_save_package (PFunArgs)
 			}
 		    }
 
-		  if (CHARTYPE(field))
+		  if (CHARTYPE (field))
 		    bprintf (query, "'");
 		}
 	    }
@@ -2522,7 +2551,7 @@ pf_database_save_package (PFunArgs)
 
 	  /* Execute the query. */
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
 	  status = gsql_query (db, query->buffer, 0);
 	  bprintf_free_buffer (query);
@@ -2531,8 +2560,11 @@ pf_database_save_package (PFunArgs)
 	    result = "true";
 	  else
 	    {
-	      if (empty_string_p (method))
-		gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
+	      if (!empty_string_p (method))
+		{
+		  errors_found++;
+		  gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
+		}
 	    }
 	}
 
@@ -2540,12 +2572,7 @@ pf_database_save_package (PFunArgs)
 	  ((empty_string_p (method) && result == (char *)NULL) ||
 	   (!empty_string_p (method) && (strcasecmp (method, "UPDATE") == 0))))
 	{
-
-	  /* No errors yet! */
-	  gsql_clear_error_message ();
-
-	  /* The INSERT failed for some reason, so let's try
-	     an UPDATE. */
+	  /* The INSERT failed for some reason, so let's try an UPDATE. */
 	  query = bprintf_create_buffer ();
 
 	  /* Let's try an UPDATE on this key and keyvalue:
@@ -2553,8 +2580,7 @@ pf_database_save_package (PFunArgs)
 
 	  bprintf (query, "UPDATE %s SET ", table);
 
-	  /* Comma-separated list of all the variable names
-	     and values. */
+	  /* Comma-separated list of all the variable names and values. */
 	  items_printed = 0;
 
 	  for (i = 0; (sym = symbols[i]) != (Symbol *)NULL; i++)
@@ -2573,7 +2599,7 @@ pf_database_save_package (PFunArgs)
 
 		  bprintf (query, "%s = ", gsql_field_name (field));
 
-		  if (CHARTYPE(field))
+		  if (CHARTYPE (field))
 		    bprintf (query, "'");
 
 		  if (sym->values_index != 0)
@@ -2583,8 +2609,7 @@ pf_database_save_package (PFunArgs)
 					     escape_char, truncate_p);
 		      else
 			{
-			  /* Collect up the array values
-			     into a string. */
+			  /* Collect up the array values into a string. */
 			  register int j;
 			  BPRINTF_BUFFER *tmpbuf = bprintf_create_buffer ();
 
@@ -2598,7 +2623,7 @@ pf_database_save_package (PFunArgs)
 			}
 		    }
 
-		  if (CHARTYPE(field))
+		  if (CHARTYPE (field))
 		    bprintf (query, "'");
 		}
 	    }
@@ -2612,15 +2637,13 @@ pf_database_save_package (PFunArgs)
 
 	  /* Execute the query. */
 	  if (debug_level)
-	    page_debug ("  Trying: <code>[%s]</code>", query->buffer);
+	    page_debug ("  Trying: [%s]", query->buffer);
 
-	  status = gsql_query (db, query->buffer, 0);
+	  status = gsql_query (db, query->buffer, 1);
 	  bprintf_free_buffer (query);
 
 	  if (status == GSQL_SUCCESS)
 	    result = "true";
-	  else
-	    gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
 	}
 
       if (result)
@@ -2672,8 +2695,7 @@ string_from_boolean (int bool)
 }
 
 /* Create a Meta-HTML string representation of an alist, which contains
-   table field information. 
-   */
+   table field information. */
 static char *
 alist_of_field_info (gsql_field *field, Database *db)
 {
@@ -2685,44 +2707,21 @@ alist_of_field_info (gsql_field *field, Database *db)
       tmpbuf = bprintf_create_buffer ();
 
       bprintf (tmpbuf, "(");
-      bprintf (tmpbuf, "(NAME . \"%s\")",
-	       gsql_field_name (field));
-
-      bprintf (tmpbuf, "(LENGTH . \"%d\")",
-	       gsql_field_length (field));
-
+      bprintf (tmpbuf, "(NAME . \"%s\")", gsql_field_name (field));
+      bprintf (tmpbuf, "(LENGTH . \"%d\")", gsql_field_length (field));
       bprintf (tmpbuf, "(TYPE . \"%s\")",
 	       pretty_field_type (gsql_field_type (field)));
-
       bprintf (tmpbuf, "(IS_UNIQUE . \"%s\")",
-	       string_from_boolean
-	       (gsql_field_is_unique (field)));
-
+	       string_from_boolean (gsql_field_is_unique (field)));
       bprintf (tmpbuf, "(IS_NULLABLE . \"%s\")",
-	       string_from_boolean
-	       (! (gsql_field_is_not_null (field))));
-
-      bprintf (tmpbuf, "(QUALIFIER . \"%s\")",
-	       gsql_field_qualifier (field));
-
-      bprintf (tmpbuf, "(OWNER . \"%s\")",
-	       gsql_field_owner (field));
-
-      bprintf (tmpbuf, "(TYPENAME . \"%s\")",
-	       gsql_field_typename (field));
-
-      bprintf (tmpbuf, "(PRECISION . \"%d\")",
-	       gsql_field_precision (field));
-
-      bprintf (tmpbuf, "(SCALE . \"%d\")",
-	       gsql_field_scale (field));
-
-      bprintf (tmpbuf, "(RADIX . \"%d\")",
-	       gsql_field_radix (field));
-
-      bprintf (tmpbuf, "(TABLE . \"%s\")",
-	       gsql_field_table (field));
-
+	       string_from_boolean (! (gsql_field_is_not_null (field))));
+      bprintf (tmpbuf, "(QUALIFIER . \"%s\")", gsql_field_qualifier (field));
+      bprintf (tmpbuf, "(OWNER . \"%s\")", gsql_field_owner (field));
+      bprintf (tmpbuf, "(TYPENAME . \"%s\")", gsql_field_typename (field));
+      bprintf (tmpbuf, "(PRECISION . \"%d\")", gsql_field_precision (field));
+      bprintf (tmpbuf, "(SCALE . \"%d\")", gsql_field_scale (field));
+      bprintf (tmpbuf, "(RADIX . \"%d\")", gsql_field_radix (field));
+      bprintf (tmpbuf, "(TABLE . \"%s\")", gsql_field_table (field));
       bprintf (tmpbuf, ")");
     }
   
@@ -2739,7 +2738,6 @@ alist_of_field_info (gsql_field *field, Database *db)
   return (val);
 }
 
-
 /* List all the column information for a table. This must be done
    with a database already open, i.e., within the scope of a
    <msql::with-open-database>.
@@ -2750,10 +2748,7 @@ alist_of_field_info (gsql_field *field, Database *db)
 
    Each alist contains 
    ((name . "column_name") (length . "max byte length") (type . "sqlDatatype")
-    (is_unique . "true") (is_not_null . "true"))
-
-*/
-
+    (is_unique . "true") (is_not_null . "true")) */
 static void
 pf_database_columns_info (PFunArgs)
 {
@@ -2832,9 +2827,7 @@ pf_database_columns_info (PFunArgs)
     columns and selecting the specified one. It is thus more
     efficient, if you are looking for info on several columns, to use
     database-columns-info, and get all the column information in a
-    single array.
-
- */
+    single array. */
 static void
 pf_database_column_info (PFunArgs)
 {
@@ -2896,7 +2889,6 @@ pf_database_column_info (PFunArgs)
 
    ACTION can be one of COMMIT or ROLLBACK.
    If unspecified, ACTION defaults to COMMIT.  */
-
 static void
 pf_sql_transact (PFunArgs)
 {
@@ -2928,9 +2920,7 @@ pf_sql_transact (PFunArgs)
   xfree (action_arg);
 }
 
-/* Convert a gsql_result struct into a meta-html alist.
-
- */
+/* Convert a gsql_result struct into a Meta-HTML alist. */
 static char *
 alist_from_row (gsql_result *query_result)
 {
@@ -3012,14 +3002,14 @@ pf_database_tables_info (PFunArgs)
 	{
 	  /* This returns a result set, with one row of info 
 	     for each table in the database. */
-	  gsql_result *result = gsql_db_list_tables
-	                            (db, qualifier, owner,name, type);
+	  gsql_result *result =
+	    gsql_db_list_tables (db, qualifier, owner,name, type);
 
 	  if (result != (gsql_result *)NULL)
 	    {
 	      register int i;
-	      char *varname;
 	      char **alists;
+	      char *varname;
 	      int rows;
 	      int arraysize = 10;
 
