@@ -29,8 +29,16 @@
    Santa Barbara, CA
    93101  */
 
+/* This version of Meta-HTML can handle dynamically loaded modules. */
+#define PACKAGE_INITIALIZER_EXTRA_CODE \
+  pagefunc_set_variable ("mhtml::module-capable", "true");
+
 #include "language.h"
-#include <dlfcn.h>
+#if defined (USE_SHL_LOAD)
+#  include <dl.h>
+#else
+#  include <dlfcn.h>
+#endif
 
 #if !defined (RTLD_NOW)
 #  define RTLD_NOW 1
@@ -38,28 +46,15 @@
 
 static void pf_load_module (PFunArgs);
 
-static PFunDesc function_table[] = {
+static PFunDesc func_table[] =
+{
   { "LOAD-MODULE",	0, 0, pf_load_module },
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
 
 PACKAGE_INITIALIZER (initialize_module_functions)
-void
-initialize_module_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
 
-  for (i = 0; function_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, function_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&function_table[i]);
-    }
-
-  /* This version of Meta-HTML can handle dynamically loaded modules. */
-  pagefunc_set_variable ("mhtml::module-capable", "true");
-}
+DOC_SECTION (DYNAMIC-MODULES)
 
 static char *static_dirs[3] =
  { "/opt/metahtml/lib", "/opt/metahtml/modules", (char *)NULL };
@@ -75,10 +70,14 @@ fully_qualified_module_name (char *name)
       static char buffer[2048];
       char **dirs = symbol_get_values ("mhtml::module-directories");
       struct stat finfo;
-      int add_so = 0;
+      int add_extension = 0;
 
-      if ((namelen < 3) || (strcmp (name + (namelen - 3), ".so") != 0))
-	add_so++;
+      {
+	char *temp = strrchr (name, '.');
+
+	if (!temp)
+	  add_extension++;
+      }
 
       if (dirs == (char **)NULL)
 	dirs = &static_dirs[0];
@@ -86,7 +85,14 @@ fully_qualified_module_name (char *name)
       for (i = 0; dirs[i] != (char *)NULL; i++)
 	{
 	  sprintf (buffer, "%s/%s", dirs[i], name);
-	  if (add_so) strcat (buffer, ".so");
+	  if (add_extension)
+	    {
+#if defined (USE_SHL_LOAD)
+	      strcat (buffer, ".O");
+#else
+	      strcat (buffer, ".so");
+#endif
+	    }
 
 	  if (stat (buffer, &finfo) == 0)
 	    return (strdup (buffer));
@@ -96,8 +102,14 @@ fully_qualified_module_name (char *name)
   return (strdup (name));
 }
 
-#define module_recent_error(file) \
+#if defined (USE_SHL_LOAD)
+extern int errno;
+#  define module_recent_error(file) \
+	page_syserr ("LOAD-MODULE: (%s) %s", file, strerror (errno))
+#else
+#  define module_recent_error(file) \
 	page_syserr ("LOAD-MODULE: (%s) %s", file, dlerror ())
+#endif
 
 #define MODULE_SYM_NAME "mhtml::loaded-modules"
 
@@ -127,7 +139,11 @@ pf_load_module (PFunArgs)
 
       if (result == (char *)NULL)
 	{
+#if defined (USE_SHL_LOAD)
+	  void *handle = (void *)shl_load (fqn, BIND_IMMEDIATE, 0L);
+#else
 	  void *handle = (void *)dlopen (fqn, RTLD_NOW);
+#endif
 
 	  if (handle != (void *)NULL)
 	    {
@@ -146,8 +162,14 @@ pf_load_module (PFunArgs)
 #endif
 	      if (no_init == (char *)NULL)
 		{
-		  VFUN *initfunc = (VFUN *)
-		    dlsym (handle, "module_initialize");
+		  VFUN *initfunc = (VFUN *)NULL;
+
+#if defined (USE_SHL_LOAD)
+		  shl_findsym (&handle, "module_initialize", TYPE_PROCEDURE,
+			       &initfunc);
+#else
+		  initfunc = (VFUN *) dlsym (handle, "module_initialize");
+#endif
 
 		  if (initfunc != (VFUN *)NULL)
 		    {
@@ -157,7 +179,11 @@ pf_load_module (PFunArgs)
 		  else
 		    {
 		      module_recent_error (fqn);
+#if defined (USE_SHL_LOAD)
+		      shl_unload (handle);
+#else
 		      dlclose (handle);
+#endif
 		    }
 		}
 	      else
