@@ -35,9 +35,7 @@
 #define DEFAULT_SQL_PREFIX_TABLENAMES 0
 
 
-PACKAGE_INITIALIZER (initialize_msql_functions)
-
-PFunDesc msqlfunc_table[] =
+static PFunDesc func_table[] =
 {
   { "MSQL::WITH-OPEN-DATABASE",		1, 0, pf_with_open_database },
   { "MSQL::DATABASE-EXEC-QUERY",	0, 0, pf_database_exec_query },
@@ -65,23 +63,7 @@ PFunDesc msqlfunc_table[] =
   { (char *)NULL,			0, 0, (PFunHandler *)NULL }
 };
 
-static void initialize_database_stack (void);
-
-void
-initialize_msql_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; msqlfunc_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, msqlfunc_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&msqlfunc_table[i]);
-    }
-
-  initialize_database_stack ();
-}
+PACKAGE_INITIALIZER (initialize_msql_functions)
 
 /****************************************************************
  * The Database object:
@@ -184,7 +166,6 @@ typedef struct
 
 static void free_database_cursors (Database *db);
 
-
 static void
 free_database_resources (Database *db)
 {
@@ -212,15 +193,10 @@ gsql_save_error_message (Database *db, char *msg)
   msql_error_index++;
 
   if (msg == GSQL_DEFAULT_ERRMSG)
-    {
-      pagefunc_set_variable (error_variable, msqlErrMsg);
-      page_debug ("MSQL: %s", msqlErrMsg);
-    }
-  else
-    {
-      pagefunc_set_variable (error_variable, msg);
-      page_debug ("MSQL: %s", msg);
-    }
+    msg = (char *)&msqlErrMsg;
+
+  pagefunc_set_variable (error_variable, msg);
+  page_debug ("MSQL: %s", msg);
 }
 
 static int
@@ -265,6 +241,7 @@ gsql_query (Database *db, char *query, int save_errors_p)
     {
       if (save_errors_p)
 	gsql_save_error_message (db, "msqlQuery");
+
       return (GSQL_ERROR);
     }
   else
