@@ -113,6 +113,10 @@ extern int errno;
 #  define xfree(x) if (x) free (x)
 #endif
 
+/* The name of the file most recently loaded with page_read_template (). */
+char *page_last_read_filename = (char *)NULL;
+static int plrf_size = 0;
+
 /* Make PAGE contain CONTENTS, and nothing else. */
 void
 page_set_contents (PAGE *page, char *contents)
@@ -205,6 +209,13 @@ page_read_template (char *passed_filename)
   if ((result != -1) && (!S_ISDIR (finfo.st_mode)))
     {
       int fd;
+      int filename_len = strlen (filename);
+
+      if (filename_len + 2 > plrf_size)
+	page_last_read_filename = 
+	  (char *)xrealloc (page_last_read_filename,
+			    (plrf_size = (20 + filename_len)));
+      strcpy (page_last_read_filename, filename);
 
       fd = os_open (filename, O_RDONLY, 0666);
       if (fd > -1)
@@ -253,6 +264,7 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
   static int *replacement_offsets = (int *)NULL;
   static int ro_slots = 0;
   int ro_index = 0;
+  int eflags = 0;
 
   regcomp (&re, this, REG_EXTENDED);
 
@@ -286,7 +298,7 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
   while (!done)
     {
       matched =
-	regexec (&re, page->buffer + search_point, 10, offsets, 0) == 0;
+	regexec (&re, page->buffer + search_point, 10, offsets, eflags) == 0;
 
       if (matched)
 	{
@@ -303,6 +315,10 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
 	      continue;
 	    }
 
+	  /* Use REG_NOTBOL for subsequent calls, thus disallowing
+	     beginning-of-line matches for those calls. */
+	  eflags = REG_NOTBOL;
+
 	  if (ro_index == 0)
 	    {
 	      if (end < point)
@@ -1303,3 +1319,44 @@ page_simple_tag_extract (PAGE *page, char *tag, int *startp, int *endp)
 
   return (result);
 }
+
+static PagePDL *page_pdl = (PagePDL *)NULL;
+int page_pdl_index = 0;
+int page_pdl_size = 0;
+
+void
+page_push_page (PAGE *page, int start, int *search_start_modified)
+{
+  if (page_pdl_index + 2 > page_pdl_size)
+    page_pdl = (PagePDL *)
+      xrealloc (page_pdl, (page_pdl_size += 10) * sizeof (PagePDL));
+  page_pdl[page_pdl_index].page = page;
+  page_pdl[page_pdl_index].start = start;
+  page_pdl[page_pdl_index].search_start_modified = search_start_modified;
+  page_pdl_index++;
+}
+
+void
+page_pop_page (void)
+{
+  page_pdl_index--;
+
+  if (page_pdl_index < 0)
+    abort ();
+}
+
+PagePDL *
+page_pdl_tos (void)
+{
+  if (page_pdl_index)
+    return (&page_pdl[page_pdl_index - 1]);
+  else
+    return ((PagePDL *)NULL);
+}
+
+PagePDL *
+page_pdl_page (int offset)
+{
+  return (&page_pdl[offset]);
+}
+
