@@ -40,6 +40,9 @@ char *metahtml_copyright_string = "\
 Copyright (C) 1995, 1996, Brian J. Fox\n\
 Copyright (C) 1996, 1997 Universal Access Inc.";
 
+/* The parser's idea of what the current line number is. */
+int parser_current_lineno = 0;
+
 /* Globally known variable holds onto to the reserved words. */
 Package *mhtml_function_package = (Package *)NULL;
 
@@ -66,6 +69,12 @@ page_return_this_page (PAGE *page)
 static PAGE *ThePage = (PAGE *)NULL;
 static int TheOffset = 0;
 
+PAGE *
+mhtml_top_level_page (void)
+{
+  return (ThePage);
+}
+
 PageEnv *
 pagefunc_save_environment (void)
 {
@@ -104,6 +113,9 @@ int mhtml_decimal_places = 2;
 /* Gets 1 when mhtml::parser-trace-mode has a value, 0 otherwise. */
 int mhtml_parser_trace_mode = 0;
 
+/* Gets 1 when mhtml::gather-documentation has a value, 0 otherwise. */
+int mhtml_gather_documentation = 0;
+
 #if defined (METAHTML_PROFILER)
 /* Gets 1 when mhtml::profile-functions has a value, 0 otherwise. */
 int mhtml_profiling_functions = 0;
@@ -120,6 +132,8 @@ pagefunc_initialize_notifiers (void)
   symbol_notify_value (sym, &mhtml_decimal_notify);
   sym = symbol_intern ("mhtml::parser-trace-mode");
   symbol_notify_value (sym, &mhtml_parser_trace_mode);
+  sym = symbol_intern ("mhtml::gather-documentation");
+  symbol_notify_value (sym, &mhtml_gather_documentation);
 #if defined (METAHTML_PROFILER)
   sym = symbol_intern ("mhtml::profile-functions");
   symbol_notify_value (sym, &mhtml_profiling_functions);
@@ -801,7 +815,7 @@ void
 page_process_page_internal (PAGE *page)
 {
   register int i, c;
-  int search_start = 0;
+  int search_start = 0, search_start_modified = -1;
   int done = 0;
   int semicolon_comments;
   static char *fname = (char *)NULL;
@@ -818,11 +832,20 @@ page_process_page_internal (PAGE *page)
 
       semicolon_comments = !mhtml_inhibit_comment_parsing;
 
+      if (search_start_modified != -1)
+	{
+	  search_start = search_start_modified;
+	  search_start_modified = -1;
+	}
+
       for (i = search_start; i < page->bindex; i++)
 	{
 	  if ((page->buffer[i] == LEFT_BRACKET) || (page->buffer[i] == '<'))
 	    break;
 
+	  if (page->buffer[i] == '\n')
+	    parser_current_lineno++;
+
 	  /* If there is a semicolon comment here, ignore it now. */
 	  if (semicolon_comments && page->buffer[i] == ';')
 	    {
@@ -831,11 +854,23 @@ page_process_page_internal (PAGE *page)
 		  (page->buffer[i + 2] == ';'))
 		{
 		  int marker = i;
+
+		get_another_comment:
 		  i += 3;
 		  while (i < page->bindex &&
 			 (!return_sequence (page->buffer[i],
 					    page->buffer[i + 1])))
 		    i++;
+
+		  parser_current_lineno++;
+
+		  /* If the very next item is another semi-colon comment,
+		     delete it in one chunk with the current one. */
+		  if (((i + 2) < page->bindex) &&
+		      (page->buffer[i + 1] == ';') &&
+		      (page->buffer[i + 2] == ';'))
+		    goto get_another_comment;
+
 		  bprintf_delete_range (page, marker, i + 1);
 		  i = marker - 1;
 		}
@@ -939,8 +974,12 @@ page_process_page_internal (PAGE *page)
 		{
 		  /* The MTHML programmer didn't close the opener correctly.
 		     Tell them, and then ignore the text. */
-		  page_debug ("Closing tag missing for %c%s ...%c",
+		  char *thisfile = pagefunc_get_variable
+		    ("*parser*::filename-pdl[0]");
+		  page_debug ("%s: Closing tag missing for %c%s ...%c",
+			      thisfile ? thisfile : "???",
 			      LEFT_BRACKET, desc->tag, RIGHT_BRACKET);
+
 		  if (syntax_checking)
 		    {
 		      syntax_failure = 1;
@@ -974,7 +1013,8 @@ page_process_page_internal (PAGE *page)
 		  memmove (open_body, open_body + 1 + fname_len,
 			   (open_body_len - (1 + fname_len)));
 
-		  vars = pagefunc_snarf_vars (open_body, uf ? 0 : 1);
+		  vars = pagefunc_snarf_vars
+		    (open_body, uf ? (uf->flags & user_ACCEPT_KEYWORDS) : 1);
 
 		  if (!complexp)
 		    {
@@ -1081,53 +1121,54 @@ page_process_page_internal (PAGE *page)
 			if (mhtml_profiling_functions)
 			  gettimeofday (&start_time, (struct timezone *)NULL);
 #endif
-			
-		      if (uf)
-			mhtml_execute_function
-			  (uf, page, body, vars, start, end, &search_start,
-			   desc->debug_level, open_body);
-		      else
-			(*desc->fun)
-			  (page, body, vars, start, end, &search_start,
-			   desc->debug_level);
+			page_push_page (page, start, &search_start_modified);
+			if (uf)
+			  mhtml_execute_function
+			    (uf, page, body, vars, start, end, &search_start,
+			     desc->debug_level, open_body);
+			else
+			  (*desc->fun)
+			    (page, body, vars, start, end, &search_start,
+			     desc->debug_level);
+			page_pop_page ();
 #if defined (METAHTML_PROFILER)
-		      if (mhtml_profiling_functions &&
-			  ((start_time.tv_usec != 0) &&
-			   (start_time.tv_sec != 0)))
-			{
-			  double usecs_start, usecs_end, elapsed;
-			  gettimeofday (&end_time, (struct timezone *)NULL);
-			  usecs_start = (double)((start_time.tv_sec * 1.0e6)
-						 + start_time.tv_usec);
-			  usecs_end = (double)((end_time.tv_sec * 1.0e6)
-					       + end_time.tv_usec);
-			  elapsed = usecs_end - usecs_start;
-			  
-			  if (uf)
-			    {
-			      if (!uf->profile_info)
-				{
-				  uf->profile_info = (PROFILE_INFO *)
-				    xmalloc (sizeof (PROFILE_INFO));
-				  uf->profile_info->usecs_spent = 0.0;
-				  uf->profile_info->times_called = 0;
-				}
-			      uf->profile_info->times_called++;
-			      uf->profile_info->usecs_spent += elapsed;
-			    }
-			  else
-			    {
-			      if (!desc->profile_info)
-				{
-				  desc->profile_info = (PROFILE_INFO *)
-				    xmalloc (sizeof (PROFILE_INFO));
-				  desc->profile_info->usecs_spent = 0.0;
-				  desc->profile_info->times_called = 0;
-				}
-			      desc->profile_info->times_called++;
-			      desc->profile_info->usecs_spent += elapsed;
-			    }
-			}
+			if (mhtml_profiling_functions &&
+			    ((start_time.tv_usec != 0) &&
+			     (start_time.tv_sec != 0)))
+			  {
+			    double usecs_start, usecs_end, elapsed;
+			    gettimeofday (&end_time, (struct timezone *)NULL);
+			    usecs_start = (double)((start_time.tv_sec * 1.0e6)
+						   + start_time.tv_usec);
+			    usecs_end = (double)((end_time.tv_sec * 1.0e6)
+						 + end_time.tv_usec);
+			    elapsed = usecs_end - usecs_start;
+
+			    if (uf)
+			      {
+				if (!uf->profile_info)
+				  {
+				    uf->profile_info = (PROFILE_INFO *)
+				      xmalloc (sizeof (PROFILE_INFO));
+				    uf->profile_info->usecs_spent = 0.0;
+				    uf->profile_info->times_called = 0;
+				  }
+				uf->profile_info->times_called++;
+				uf->profile_info->usecs_spent += elapsed;
+			      }
+			    else
+			      {
+				if (!desc->profile_info)
+				  {
+				    desc->profile_info = (PROFILE_INFO *)
+				      xmalloc (sizeof (PROFILE_INFO));
+				    desc->profile_info->usecs_spent = 0.0;
+				    desc->profile_info->times_called = 0;
+				  }
+				desc->profile_info->times_called++;
+				desc->profile_info->usecs_spent += elapsed;
+			      }
+			  }
 #endif /* METAHTML_PROFILER */
 		      }
 			      
@@ -1184,7 +1225,13 @@ mhtml_evaluate_string (char *body)
 
   evaluated = page_create_page ();
   page_set_contents (evaluated, body);
-  page_process_page_internal (evaluated);
+
+  {
+    int lineno = parser_current_lineno;
+    page_process_page_internal (evaluated);
+    parser_current_lineno = lineno;
+  }
+    
   result = evaluated->buffer;
   free (evaluated);
 
@@ -1250,7 +1297,7 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 	wrapper_packname = strdup ("local");
     }
 
-  /* Gather named arguments if present. */
+  /* Gather named parameters if present. */
   {
     register int i;
     char *param;
@@ -1294,7 +1341,19 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
   else
     {
       uf->type = type;
-      if (uf->packname) free (uf->packname);
+      xfree (uf->packname);
+
+      if (mhtml_gather_documentation && uf->documentation)
+	{
+	  register int i;
+
+	  for (i = 0; uf->documentation[i] != (char *)NULL; i++)
+	    free (uf->documentation[i]);
+
+	  free (uf->documentation);
+	  uf->documentation = (char **)NULL;
+	}
+
       uf->packname = wrapper_packname;
       if (uf->named_parameters)
 	{
@@ -1304,6 +1363,7 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 	    free (uf->named_parameters[i]);
 	  free (uf->named_parameters);
 	}
+
       uf->named_parameters = named_parameters;
 
       free (uf->body);
@@ -1312,10 +1372,22 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 
   uf->flags = 0;
 
+  /* Can the caller supply keyword parameters for this function? */
+  if (var_present_p (vars, "&key"))
+    uf->flags |= user_ACCEPT_KEYWORDS;
+
   /* Strip comments now, rather than later. */
   {
     register int i = 0;
     char *b = uf->body;
+    int passed_documentation = 1;
+    char **docstrings = (char **)NULL;
+    int docstrings_index = 0;
+    int docstrings_size = 0;
+
+    /* Perhaps remember leading comments as documentation. */
+    if (mhtml_gather_documentation)
+      passed_documentation = 0;
 
     while (b[i] != '\0')
       {
@@ -1324,11 +1396,34 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 	    register int j;
 	    
 	    for (j = i; (b[j] != '\0') && (b[j] != '\n'); j++);
+
+	    if (!passed_documentation)
+	      {
+		int start = i + 3, len = j - start;
+
+		if (docstrings_index + 2 > docstrings_size)
+		  docstrings = (char **)xrealloc
+		    (docstrings, (docstrings_size += 8) * sizeof (char *));
+
+		docstrings[docstrings_index] = (char *)xmalloc (1 + len);
+		strncpy (docstrings[docstrings_index], b + start, len);
+		docstrings[docstrings_index][len] = '\0';
+		docstrings_index++;
+		docstrings[docstrings_index] = (char *)NULL;
+	      }
+
 	    memmove (b + i, b + j, strlen (b + j) + 1);
 	  }
 	else
-	  i++;
+	  {
+	    if (!passed_documentation && !whitespace (b[i]))
+	      passed_documentation++;
+
+	    i++;
+	  }
       }
+
+    uf->documentation = docstrings;
   }
      
   /* If the user wants special behaviour for the whitespace present in
@@ -1389,11 +1484,10 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 		  break;
 
 		case '\r':
-		  if (b[i + 1] == '\n')
+		  if ((b[i + 1] == '\n') || (b[i + 1] == '\0'))
 		    {
 		      if (!quote_level && !brace_level)
 			{
-			  i++;
 			  start = i;
 			  while (whitespace (b[i])) i++;
 			  memmove (b + start, b + i, 1 + strlen (b + i));
@@ -1420,6 +1514,102 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
     }
 }
 
+char *
+quote_for_setvar (char *value)
+{
+  register int j = 0, k = 1;
+  static char *setval = (char *)NULL;
+  static int setval_size = 0;
+  int max_space_needed = (2 * strlen (value)) + 4;
+
+  if (max_space_needed > setval_size)
+    setval = (char *)xrealloc (setval, (setval_size = max_space_needed));
+
+  setval[0] = '"';
+
+  if (value != (char *)NULL)
+    for (j = 0, k = 1; value[j] != '\0'; j++)
+      {
+	if ((value[j] == '"') || (value[j] == '\\'))
+	  setval[k++] = '\\';
+	
+	setval[k++] = value[j];
+      }
+
+  setval[k++] = '"';
+  setval[k] = '\0';
+
+  return (setval);
+}
+
+void
+mhtml_let (PFunArgs)
+{
+  register int i = 0;
+  Package *saved_vars = symbol_get_package ((char *)NULL);
+  Package *pack = CurrentPackage;
+  char *packname = pack && pack->name ? strdup (pack->name) : (char *)NULL;
+  Symbol **symbols = symbols_of_package (vars);
+  int jump_again = 0;
+
+  /* Save the current variables in our local package, and bind those
+     variables to their new values. */
+  for (i = 0; ((symbols != (Symbol **)NULL) &&
+	       (symbols[i] != (Symbol *)NULL)); i++)
+    {
+      Symbol *from = symbols[i];
+      Symbol *sym = symbol_intern_in_package (pack, from->name);
+
+      symbol_copy (sym, saved_vars);
+      symbol_copy (from, pack);
+    }
+
+  /* Execute the body. */
+  {
+    PageEnv *page_environ;
+    PAGE *body_code = page_copy_page (body);
+
+    page_environ = pagefunc_save_environment ();
+
+    if ((jump_again = setjmp (page_jmp_buffer)) == 0)
+      page_process_page_internal (body_code);
+    pagefunc_restore_environment (page_environ);
+
+    if (body_code != (PAGE *)NULL)
+      {
+	if (!jump_again && (body_code->buffer != (char *)NULL))
+	  {
+	    PagePDL *tpage = page_pdl_tos ();
+	    if ((tpage != (PagePDL *)NULL) && (tpage->page == page) &&
+		(*tpage->search_start_modified != -1))
+	      {
+		start = *tpage->search_start_modified;
+		*tpage->search_start_modified = -1;
+	      }
+
+	    bprintf_insert (page, start, "%s", body_code->buffer);
+	    *newstart = start + (body_code->bindex);
+	  }
+
+	page_free_page (body_code);
+      }
+  }
+
+  /* If the package had a name before, get the named package again, because
+     the body code might have deleted it!  If it had no name, there isn't
+     any way that the body code could delete it, so trust that it exists. */
+  if (packname)
+    {
+      pack = symbol_get_package (packname);
+      free (packname);
+    }
+
+  symbol_copy_package (saved_vars, pack);
+  symbol_destroy_package (saved_vars);
+
+  if (jump_again) longjmp (page_jmp_buffer, 1);
+}
+
 /* Execute the subst, function or macro described by UF. */
 void
 mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
@@ -1451,15 +1641,8 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 
 		  if (!empty_string_p (arg))
 		    {
-		      char *insertion = mhtml_evaluate_string (arg);
-
-		      if (!empty_string_p (insertion))
-			{
-			  bprintf_insert (subber, i, "%s", insertion);
-			  i += strlen (insertion);
-			}
-
-		      if (insertion) free (insertion);
+		      bprintf_insert (subber, i, "%s", arg);
+		      i += strlen (arg);
 		    }
 		}
 	      else if (subber->buffer[i] == '\\')
@@ -1491,32 +1674,15 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 
 		  if (!empty_string_p (body->buffer))
 		    {
-		      register int k;
 		      char *setval;
 
 		      j = 0;
 		      if (uf->type != user_MACRO)
 			while (whitespace (body->buffer[j])) j++;
 
-		      setval = (char *)xmalloc ((2 * body->bindex) + 4);
-
-		      setval[0] = '"';
-
-		      for (k = 1; j < body->bindex; j++)
-			{
-			  if ((body->buffer[j] == '\\') ||
-			      (body->buffer[j] == '"'))
-			    setval[k++] = '\\';
-
-			  setval[k++] = body->buffer[j];
-			}
-
-		      setval[k++] = '"';
-		      setval[k] = '\0';
-
+		      setval = quote_for_setvar (body->buffer + j);
 		      bprintf_insert (subber, i, "%s", setval);
-		      free (setval);
-		      i += k;
+		      i += strlen (setval);
 		    }
 		}
 	      else if (((subber->bindex - i) > 4) &&
@@ -1562,54 +1728,135 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 	    }
 	}
 
+      /* Done doing textual replacement in BODY.  If there is anything left,
+	 then handle named parameters, etc. */
       if (!empty_string_p (subber->buffer))
 	{
 	  char *packname = uf->packname;
-	  char *parameter_setter = (char *)NULL;
+	  Package *param_package = (Package *)NULL;
 
 	  if (uf->named_parameters != (char **)NULL)
 	    {
-	      BPRINTF_BUFFER *wrapper = bprintf_create_buffer ();
-	      bprintf (wrapper, "%cset-var", LEFT_BRACKET);
+	      int gathering_optional = 0;
+	      int gathering_keys = 0;
+	      int arg_offset = 0;
+
+	      if ((packname != (char *)NULL) &&
+		  (strcmp (packname, "local") == 0))
+		param_package = symbol_get_package ((char *)NULL);
+	      else
+		param_package = symbol_get_package (packname);
+
 	      for (i = 0; uf->named_parameters[i] != (char *)NULL; i++)
 		{
-		  char *value = get_positional_arg (vars, i);
+		  char *param = uf->named_parameters[i];
+		  char *value = (char *)NULL;
+
+		  /* Handle &optional argument. */
+		  if (strcasecmp (param, "&optional") == 0)
+		    {
+		      gathering_optional++;
+		      continue;
+		    }
+
+		  /* Handle &key argument. */
+		  if (strcasecmp (param, "&key") == 0)
+		    {
+		      gathering_keys++;
+		      continue;
+		    }
+
+		  /* Handle &rest argument. */
+		  if (strcasecmp (param, "&rest") == 0)
+		    {
+		      param = uf->named_parameters[i + 1];
+
+		      if (param != (char *)NULL)
+			{
+			  Symbol *sym;
+			  char *tem = param;
+
+			  i++;
+			  param = symbol_canonical_name (param);
+			  sym = symbol_intern_in_package
+			    (param_package, param);
+
+			  if (strcmp (param, tem) != 0)
+			    {
+			      int array_offset = 0;
+
+			      while ((value = get_positional_arg
+				      (vars, arg_offset++))
+				     != (char *)NULL)
+				{
+				  char *setval = value;
+
+				  /* if (uf->type == user_DEFUN) */
+				    {
+				      setval = mhtml_evaluate_string (setval);
+				      symbol_add_value (sym, setval);
+				      free (setval);
+				    }
+				    /*
+				  else
+				    {
+				      symbol_add_value (sym, setval);
+				    }
+				    */
+				  array_offset++;
+				}
+			    }
+			  else
+			    {
+			      BPRINTF_BUFFER *gather = bprintf_create_buffer();
+
+			      while ((value = get_positional_arg
+				      (vars, arg_offset++))
+				     != (char *)NULL)
+				bprintf (gather, "%s ", value);
+
+			      if (gather->bindex)
+				{
+				  char *setval = gather->buffer;
+
+				  /* if (uf->type == user_DEFUN) */
+				    {
+				      setval = mhtml_evaluate_string (setval);
+				      symbol_add_value (sym, setval);
+				      free (setval);
+				    }
+				    /* else symbol_add_value (sym, setval); */
+				}
+
+			      bprintf_free_buffer (gather);
+			    }
+
+			  free (param);
+			  break;
+			}
+		      else
+			param = uf->named_parameters[i];
+		    }
+
+		  /* Get argument value of the right type. */
+		  if (gathering_keys)
+		    value = forms_get_tag_value_in_package (vars, param);
+		  else
+		    value = get_positional_arg (vars, arg_offset++);
 
 		  if (value != (char *)NULL)
 		    {
-		      register int k;
-		      char *setval;
-
-		      if (uf->type == user_DEFUN)
-			value = mhtml_evaluate_string (value);
-
-		      setval = (char *)xmalloc ((2 * strlen (value)) + 4);
-
-		      setval[0] = '"';
-
-		      for (j = 0, k = 1; value[j] != '\0'; j++)
+		      /* if (uf->type == user_DEFUN) */
 			{
-			  if ((value[j] == '"') || (value[j] == '\\'))
-			    setval[k++] = '\\';
-
-			  setval[k++] = value[j];
+			  value = mhtml_evaluate_string (value);
+			  forms_set_tag_value_in_package
+			    (param_package, param, value);
+			  free (value);
 			}
-
-		      setval[k++] = '"';
-		      setval[k] = '\0';
-
-		      bprintf (wrapper, " %s=%s",
-			       uf->named_parameters[i], setval);
-
-		      if (uf->type == user_DEFUN)
-			free (value);
-
-		      free (setval);
+			/* else forms_set_tag_value_in_package
+			   (param_package, param, value); */
 		    }
 		}
-	      bprintf (wrapper, "%c", RIGHT_BRACKET);
-	      parameter_setter = wrapper->buffer;
-	      free (wrapper);
 	    }
 
 	  if (packname)
@@ -1618,19 +1865,18 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 	      if (strcmp (packname, "local") == 0)
 		{
 		  Package *current_package = CurrentPackage;
-		  char *packname = current_package->name ?
+		  char *savepack = current_package->name ?
 		    strdup (current_package->name) : (char *)NULL;
+		  Package *local = param_package;
 		  char *x;
 
-		  CurrentPackage = symbol_get_package ((char *)NULL);
+		  if (local == (Package *)NULL)
+		    local = symbol_get_package ((char *)NULL);
 
-		  if (parameter_setter)
-		    {
-		      x = mhtml_evaluate_string (parameter_setter);
-		      xfree (x);
-		    }
+		  symbol_push_package (local);
 
 		  x = mhtml_evaluate_string (subber->buffer);
+
 		  if (!empty_string_p (x))
 		    {
 		      bprintf_insert (page, start, "%s", x);
@@ -1638,30 +1884,46 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 		    }
 
 		  xfree (x);
-		  if (packname)
+		  symbol_pop_package ();
+
+		  if (savepack)
 		    {
-		      current_package = symbol_get_package (packname);
-		      free (packname);
+		      current_package = symbol_get_package (savepack);
+		      free (savepack);
 		    }
 
 		  CurrentPackage = current_package;
 		}
 	      else
 		{
-		  bprintf_insert (page, start,
-				  "%cin-package %s%c%s%s%c/in-package%c",
-				  LEFT_BRACKET, packname, RIGHT_BRACKET,
-				  parameter_setter ? parameter_setter : "",
-				  subber->buffer,
-				  LEFT_BRACKET, RIGHT_BRACKET);
+		  Package *letpack = symbol_get_package (packname);
+		  Package *savepack = CurrentPackage;
+		  char *savename = (char *)NULL;
+
+		  if (savepack->name)
+		    savename = strdup (savepack->name);
+
+		  CurrentPackage = letpack;
+		  mhtml_let (page, subber, param_package, start, end,
+			     newstart, debug_level);
+
+		  if (savename)
+		    {
+		      CurrentPackage = symbol_get_package (savename);
+		      free (savename);
+		    }
+		  else
+		    CurrentPackage = savepack;
 		}
 	    }
 	  else
 	    {
-	      bprintf_insert (page, start, "%s%s",
-			      parameter_setter ? parameter_setter : "",
-			      subber->buffer);
+	      mhtml_let (page, subber, param_package, start,
+			 end, newstart, debug_level);
 	    }
+
+	  if (param_package)
+	    symbol_destroy_package (param_package);
 	}
 
       page_free_page (subber);
@@ -1991,3 +2253,47 @@ mhtml_base64decode (char *encoded, int *len)
 
   return (decoded);
 }
+
+int
+number_p (char *string)
+{
+  register int i;
+  int result = 0;
+
+  if (!string)
+    return (0);
+
+  /* Skip leading whitespace. */
+  for (i = 0; whitespace (string[i]); i++);
+
+  if (string[i])
+    {
+      int decimal_seen = 0;
+
+      result = 1;
+
+      if ((string[i] == '-') || (string[i] == '+'))
+	i++;
+
+      for (; string[i]; i++)
+	{
+	  if (string[i] == '.')
+	    {
+	      if (decimal_seen)
+		{
+		  result = 0;
+		  break;
+		}
+	      else
+		decimal_seen++;
+	    }
+	  else if (!isdigit (string[i]))
+	    {
+	      result = 0;
+	      break;
+	    }
+	}
+    }
+
+  return (result);
+}
