@@ -53,6 +53,7 @@ typedef struct
   char *body;		/* The body of the macro or subst. */
   char *packname;	/* Default package for the scope of this function. */
   char **named_parameters; /* Variables to bind during function invocation. */
+  char **documentation;	/* The first set of comments that follow the defun. */
 #if defined (METAHTML_PROFILER)
   PROFILE_INFO *profile_info;
 #endif
@@ -71,7 +72,31 @@ extern UserFunction *mhtml_find_user_function (char *name);
      (((c) == '\n') || (((c) == '\r') && ((c1) == '\n')))
 
 #if defined (LIBMHTML_INTERNAL)
-#define PACKAGE_INITIALIZER(name)
+#if !defined (PACKAGE_INITIALIZER_EXTRA_CODE)
+#  define PACKAGE_INITIALIZER_EXTRA_CODE
+#endif
+
+#define PACKAGE_INITIALIZER(initializer_function_name)			\
+void									\
+initializer_function_name (Package *package)				\
+{									\
+  register int i;							\
+  register Symbol *sym;							\
+									\
+  for (i = 0; func_table[i].tag != (char *)NULL; i++)			\
+    {									\
+      sym = symbol_intern_in_package (package, func_table[i].tag);	\
+      sym->type = symtype_FUNCTION;					\
+      sym->values = (char **) (&func_table[i]);				\
+    }									\
+   PACKAGE_INITIALIZER_EXTRA_CODE					\
+}
+
+#define DOC_SECTION(secname)
+#define DEFUN(name, args, documentation) static void name (PFunArgs)
+#define DEFUNX(name, args, documentation)
+#define DEFMACRO(name, args, documentation) static void name (PFunArgs)
+#define DEFMACROX(name, args, documentation)
 
 /* A pointer to a function which takes a Package as an argument, and
    initializes that package with Meta-HTML function pointers. */
@@ -89,7 +114,7 @@ extern Package *mhtml_user_keywords;
 /* The "DEFAULT" package. */
 extern Package *PageVars;
 
-/* What a pagefunc takes as arguments.
+/* What a Meta-HTML builtin function takes as arguments:
    PAGE is the entire page, as passed to page_fun_function ().
    BODY is the contents of the tag body, exclusive of the tags.
    VARS is the list of variables found in the start tag.
@@ -122,6 +147,9 @@ typedef struct
 /* Return the primitive descriptor for TAG, or NULL if there is none. */
 extern PFunDesc *pagefunc_get_descriptor (char *tag);
 
+/* Bind variables in the current package only for the duration of body. */
+extern void mhtml_let (PFunArgs);
+
 /* Execute the subst, function or macro described by UF. */
 extern void mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr);
 
@@ -142,11 +170,16 @@ extern jmp_buf page_jmp_buffer;
 #define user_SUBST 2
 #define user_DEFUN 3
 
+/* Flags that can be associated with a UserFunction. */
 #define user_WHITESPACE_DELETED	0x01
 #define user_WEAK_MACRO		0x02
+#define user_ACCEPT_KEYWORDS	0x04
 
 extern char *metahtml_copyright_string;
 
+/* The parser's idea of what the current line number is. */
+extern int parser_current_lineno;
+
 /* Gets 1 when mhtml::inhibit-comment-parsing has a value, 0 otherwise. */
 extern int mhtml_inhibit_comment_parsing;
 
@@ -180,6 +213,7 @@ extern char *get_positional_arg (Package *package, int position);
 extern char *read_sexp_1 (char *string, int *start, int stop_at_equals_p,
 			  int one_list);
 extern char *read_sexp (char *string, int *start, int stop_at_equals_p);
+extern char *quote_for_setvar (char *string);
 
 /* If you want to delete a package, you should probably call this function
    rather than calling symbol_destroy_package () from symbols.c.  This 
@@ -244,5 +278,6 @@ extern int mhtml_all_digits (char *string);
 extern char *mhtml_base64decode (char *encoded, int *len);
 
 extern PAGE *parser_top_page (void);
+extern int number_p (char *string);
 
 #endif /*_PARSER_H_ */
