@@ -38,8 +38,12 @@ static void pf_read_package_file (PFunArgs);
 static void pf_switch_user (PFunArgs);
 static void pf_the_page (PFunArgs);
 static void pf_point (PFunArgs);
+static void pf_function_documentation (PFunArgs);
+static void pf_function_arguments (PFunArgs);
+static void pf_quote_for_setvar (PFunArgs);
 
-PFunDesc prims_table[] = {
+static PFunDesc func_table[] =
+{
   { "%%EVAL",			0, 0, pf_eval },
   { "%%READ",			0, 0, pf_read },
   { "%%WRITE-PACKAGE-FILE",	0, 0, pf_write_package_file },
@@ -47,23 +51,15 @@ PFunDesc prims_table[] = {
   { "%%SWITCH-USER",		0, 0, pf_switch_user },
   { "%%THE-PAGE",		0, 0, pf_the_page },
   { "%%POINT",			0, 0, pf_point },
+  { "%%FUNCTION-DOCUMENTATION",	0, 0, pf_function_documentation },
+  { "%%FUNCTION-ARGUMENTS",	0, 0, pf_function_arguments },
+  { "%%QUOTE-FOR-SET-VAR",	0, 0, pf_quote_for_setvar },
+
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
 PACKAGE_INITIALIZER (initialize_primitive_functions)
-void
-initialize_primitive_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; prims_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, prims_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&prims_table[i]);
-    }
-}
+DOC_SECTION (PRIMITIVE-OPERATORS)
 
 static void
 pf_eval (PFunArgs)
@@ -270,3 +266,78 @@ pf_point (PFunArgs)
 {
   bprintf_insert (page, start, "%d", start);
 }
+
+/* <%%function-documentation user-function> --> documentation strings. */
+static void
+pf_function_documentation (PFunArgs)
+{
+  char *name = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (name))
+    {
+      UserFunction *uf = mhtml_find_user_function (name);
+
+      if ((uf != (UserFunction *)NULL) && (uf->documentation != (char **)NULL))
+	{
+	  register int i;
+
+	  for (i = 0; uf->documentation[i] != (char *)NULL; i++)
+	    {
+	      bprintf_insert (page, start, "%s\n", uf->documentation[i]);
+	      start += 1 + strlen (uf->documentation[i]);
+	    }
+
+	  *newstart = start;
+	}
+    }
+
+  xfree (name);
+}
+
+/* <%%function-arguments user-function> --> Argument strings. */
+static void
+pf_function_arguments (PFunArgs)
+{
+  char *name = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  if (!empty_string_p (name))
+    {
+      UserFunction *uf = mhtml_find_user_function (name);
+
+      if ((uf != (UserFunction *)NULL) &&
+	  (uf->named_parameters != (char **)NULL))
+	{
+	  register int i;
+
+	  for (i = 0; uf->named_parameters[i] != (char *)NULL; i++)
+	    {
+	      bprintf_insert (page, start, "%s\n", uf->named_parameters[i]);
+	      start += 1 + strlen (uf->named_parameters[i]);
+	    }
+
+	  *newstart = start;
+	}
+    }
+
+  xfree (name);
+}
+
+/* <%%quote-for-set-var arg...> --> "<evaluated arg>..."> */
+static void
+pf_quote_for_setvar (PFunArgs)
+{
+  if (body->buffer)
+    {
+      register int i;
+      char *value;
+
+      for (i = 0; i < body->bindex && whitespace (body->buffer[i]); i++);
+      value = mhtml_evaluate_string (body->buffer + i);
+
+      if (value != (char *)NULL)
+	{
+	  bprintf_insert (page, start, "%s", quote_for_setvar (value));
+	  free (value);
+	}
+    }
+}
