@@ -29,39 +29,7 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-extern int open (char *, int, ...);
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <sys/stat.h>
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
+#include "language.h"
 #include "symdump.h"
 
 /* The name of the variable which holds the already found filenames. */
@@ -92,26 +60,14 @@ static EXT2FILETYPE extensions[] =
 
 static void pf_require (PFunArgs);
 
-PFunDesc require_table[] =
+static PFunDesc func_table[] =
 {
   { "require",			0, 0, pf_require },
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
 PACKAGE_INITIALIZER (initialize_require_functions)
-void
-initialize_require_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; require_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, require_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&require_table[i]);
-    }
-}
+DOC_SECTION (FILE-OPERATORS)
 
 /* Return the full pathname of the file found in webspace at ROOT and LEAF,
    side-effecting FILETYPE based on the extension of that file.  For LEAFs
@@ -221,8 +177,25 @@ require_remember_pathname (char *pathname)
   symbol_add_value (sym, pathname);
 }
 
-static void
-pf_require (PFunArgs)
+DEFUN (pf_require, stem,
+"<code>require</code> tries hard to locate the source or library file
+specified by <var stem>, and then loads that file if it hasn't already
+been loaded.
+
+If the variable <varref mhtml::require-directories> is present, then
+it is an array of directory names (without trailing slashes) relative
+to Web space that should be searched through, in the order that they
+appear in the array.
+
+<code>require</code> understands the following extensions:
+
+<ul>
+<li> <b>.mhtml</b>, or <b>.src</b>: A <meta-html> source file.
+
+<li> <b>.lib</b>: A <meta-html> library file.
+</ul>
+
+<code>require</code> loads the newest version of the file that it finds, and records the complete pathname of the loaded file in the array variable <varref mhtml::require-loaded>.")
 {
   register int i;
   char *arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
@@ -272,6 +245,10 @@ pf_require (PFunArgs)
 #if defined (NOT_BINARY_COMPATIBLE)
 		bprintf_insert (page, start, "%s", file_contents->buffer);
 #else
+		bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
+				page_last_read_filename);
+		bprintf (file_contents, "<*parser*::pop-file>");
+
 		/* Manually insert the file instead of letting bprintf
 		   do it for us.  This is because the file could contain
 		   binary data, and then file->bindex wouldn't necessarily
@@ -296,7 +273,7 @@ pf_require (PFunArgs)
 
 	case require_TYPE_LIB:
 	  {
-	    int fd = open (pathname, O_RDONLY, 0666);
+	    int fd = os_open (pathname, O_RDONLY, 0666);
 	    Package *pack;
 
 	    if (fd > -1)
