@@ -90,7 +90,7 @@ static void pf_session_import_posted (PFunArgs);
    file used to store session information. */
 static void pf_set_session_db (PFunArgs);
 
-PFunDesc sessionfunc_table[] =
+static PFunDesc func_table[] =
 {
   /* The following functions deal with the session database. */
   { "CREATE-SESSION",	0, 0, pf_create_session },
@@ -120,19 +120,7 @@ PFunDesc sessionfunc_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_session_functions)
-void
-initialize_session_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; sessionfunc_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, sessionfunc_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&sessionfunc_table[i]);
-    }
-}
+DOC_SECTION (SESSION-OPERATORS)
 
 /* Get the values of VAR1 .. VARn and concatenate them to make a key.
    Create the session with that key.
