@@ -39,7 +39,8 @@ static void pf_stream_readable (PFunArgs);
 static void pf_stream_writable (PFunArgs);
 static void pf_stream_shutdown (PFunArgs);
 
-PFunDesc streamfunc_table[] = {
+static PFunDesc func_table[] =
+{
   { "WITH-OPEN-STREAM",		1, 0, pf_with_open_stream },
   { "STREAM-PUT",		0, 0, pf_stream_put },
   { "STREAM-GET",		0, 0, pf_stream_get },
@@ -51,6 +52,9 @@ PFunDesc streamfunc_table[] = {
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
+DOC_SECTION (STREAM-OPERATORS)
+PACKAGE_INITIALIZER (initialize_stream_functions)
+
 typedef enum { stream_FILE, stream_TCP, stream_CLOSED } StreamType;
 
 #define stream_MUST_SEEK 0x01
@@ -59,7 +63,8 @@ typedef enum { stream_FILE, stream_TCP, stream_CLOSED } StreamType;
 #define op_READ  1
 #define op_WRITE 2
 
-typedef struct {
+typedef struct
+{
   char *name;		/* The name used to open this stream. */
   StreamType type;	/* The type of stream: stream_FILE, stream_TCP, etc. */
   int fd;		/* The file descriptor that describes this stream. */
@@ -80,20 +85,6 @@ int open_streams_index = 0;
 extern int tcp_to_host (char *host, char *service);
 extern jmp_buf page_jmp_buffer;
 
-PACKAGE_INITIALIZER (initialize_stream_functions)
-void
-initialize_stream_functions (Package *package)
-{
-  register int i;
-  Symbol *sym;
-
-  for (i = 0; streamfunc_table[i].tag != (char *)NULL; i++)
-    {
-      sym = symbol_intern_in_package (package, streamfunc_table[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&streamfunc_table[i]);
-    }
-}
 
 /* Install this stream, and return the index which is used to identify it. */
 static int
@@ -187,8 +178,51 @@ int mhtml_stdin_fileno = 0;
 int mhtml_stdout_fileno = 1;
 int mhtml_stderr_fileno = 2;
 
-static void
-pf_with_open_stream (PFunArgs)
+DEFMACRO (pf_with_open_stream,
+	  var name &key type=(FILE|TCP) mode=OPEN-MODE notimeout=true,
+"Create an environment in which <var var> is bound to the
+<i>indicator</i> of an open stream, named by <var name>.
+
+The stream is either of type <code>FILE</code>, or <code>TCP</code>,
+in which case it is an open network connection.  When <var type> is
+<code>TCP</code>, <var name> specifies the host and port to connect
+to, in the form <code>Hostname:Port-Number</code>.  <var port-number>
+can either be a symbolic name which is widely recognized (e.g.,
+<code>SMTP</code>), or the digits which make up the port number,
+(e.g., <code>25</code>). Finally, the keyword argument <var
+notimeout=true> may be given, which indicates that the amount of time
+that <Meta-HTML> should wait during IO operations on this stream
+is infinite -- all processing will block until the stream is
+successfully read from or written to.  The default timeout is
+dependent on which specific operation is being performed, and the
+amount of data which is being read or written, but is generally
+suitable for writing data at about 14.4kbps.
+
+The possible values for <var mode> are:
+
+<ul>
+  <li> <code>READ</code>:<br>
+  The stream is opened for reading only.  The underlying object must
+already exist.
+
+  <li> <code>WRITE</code>:<br>
+  The stream is opened for writing only.  The underlying object must
+already exist.
+
+  <li> <code>READ-WRITE</code>:<br>
+  The stream is opened for both reading and writing.  The underlying
+object must already exist.
+
+  <li> <code>APPEND</code>:<br>
+  The stream is opened for writing only.  If the underlying object did
+not already exist, it is created.  Information written to this stream
+appears after any information that was already present in the
+underlying object.
+
+  <li> <code>WRITE-CREATE</code>:<br>
+  The stream is opened for writing only.  The underlying object is
+created fresh, even if it had already existed.
+</ul>")
 {
   char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *filename = mhtml_canonicalize_file_name_argument
@@ -583,8 +617,8 @@ stream_read_chunk (Stream *stream, int length)
   return (buffer);
 }
 
-static void
-pf_stream_put (PFunArgs)
+DEFUN (pf_stream_put, stream string,
+"Writes <var string> to the open <var stream>.")
 {
   Stream *stream = get_stream_reference (vars);
 
@@ -601,8 +635,11 @@ pf_stream_put (PFunArgs)
     }
 }
 
-static void
-pf_stream_put_contents (PFunArgs)
+DEFUN (pf_stream_put_contents, stream var,
+"Writes the contents of the binary variable <var var> to the open <var
+stream>.  Binary variables are generally created with <funref
+stream-operators stream-get-contents>, or when uploading a file from
+a Web browser.")
 {
   Stream *stream = get_stream_reference (vars);
 
@@ -646,8 +683,19 @@ pf_stream_put_contents (PFunArgs)
     }
 }
 
-static void
-pf_stream_get (PFunArgs)
+DEFUN (pf_stream_get, stream &key stop-at=character,
+"Reads a chunk of data from <var stream> and returns it. <var stream>
+is read until the character value specified by <var character>
+(defaulting to a newline) is reached, and that data upto and including
+the final <var character> is returned. One way to read all of the
+available data is to supply the empty string as the value of <var stop-at>: 
+
+<example>
+<with-open-stream stream www.ua.com:80 type=tcp mode=read-write>
+  <stream-put stream \"GET /welcome.mhtml HTTP/1.0\n\n\">
+  <set-var the-page = <stream-get stream stop-at=\"\">>
+</with-open-stream>
+</example>")
 {
   Stream *stream = get_stream_reference (vars);
   char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
@@ -672,8 +720,36 @@ pf_stream_get (PFunArgs)
   xfree (stop_at);
 }
 
-static void
-pf_stream_get_contents (PFunArgs)
+DEFUN (pf_stream_get_contents,
+       stream var &key chunk-size=size stop-at=character,
+"Reads the contents of the stream <var stream> into the binary
+variable <var var>.  You can easily use this in conjunction with
+<funref stream-operators stream-put-contents> and <funref variables
+coerce-var>.
+
+If <var stop-at> is supplied, it is a character at which the function
+should stop reading data from the stream.  Unlike
+<code>stream-get</code>, <var stop-at> defaults to the empty
+character, thus causing the entire stream to be read.
+
+If <var chunk-size> is supplied, it is the amount to read at one
+time.  This is useful for when you have an unbounded amount of data to
+read, and you would like to process it in manageable chunks.
+
+For example, the following code copies data from an open network
+stream to a file on the local disk, without using more than 32k of
+memory:
+
+<example>
+<with-open-stream src data-server.ua.com:2345 type=tcp mode=read>
+  <with-open-stream dst /tmp/datafile type=file mode=write-create>
+    <while <stream-readable s>>
+      <stream-get-contents src chunk chunk-size=32768>
+      <stream-put-contents dst chunk>
+    </while>
+  </with-open-stream>
+</with-open-stream>
+</example>")
 {
   Stream *stream = get_stream_reference (vars);
   char *stop_at = mhtml_evaluate_string (get_value (vars, "stop-at"));
@@ -728,8 +804,14 @@ pf_stream_get_contents (PFunArgs)
   xfree (stop_at);
 }
 
-static void
-pf_stream_readable (PFunArgs)
+DEFUN (pf_stream_readable, stream &key delay=seconds,
+"Returns \"true\" if <var stream> is an open stream which was opened
+with a mode of read, or read-write, and which still has data pending.
+
+The optional keyword <var delay> is used to specify the amount of time
+in seconds that <code>stream-readable</code> should wait for the
+stream to become ready for reading.  The default value is zero seconds;
+no waiting period is used.")
 {
   Stream *stream = get_stream_reference (vars);
   char *delay = get_value (vars, "delay");
@@ -771,8 +853,16 @@ pf_stream_readable (PFunArgs)
     }
 }
 
-static void
-pf_stream_writable (PFunArgs)
+
+DEFUN (pf_stream_writable, stream &key delay=seconds,
+"Returns \"true\" if <var stream> is an open stream which was opened
+with a mode of write, append, read-write, or write-create, and which
+is available to have data written to it.
+
+The optional keyword <var delay> is used to specify the amount of time
+in seconds that <code>stream-writable</code> should wait for the
+stream to become ready for writing.  The default value is zero seconds;
+no waiting period is used.")
 {
   Stream *stream = get_stream_reference (vars);
   char *delay = get_value (vars, "delay");
@@ -814,8 +904,16 @@ pf_stream_writable (PFunArgs)
     }
 }
 
-static void
-pf_stream_shutdown (PFunArgs)
+DEFUN (pf_stream_shutdown, stream,
+"Only for use with network streams, this tells the underlying
+operating system (and the other end of the network connection) that no
+additional input or output will be done using this object. In effect,
+it immediately closes the stream.
+
+When used in conjunction with a network stream opened on
+<code>*standard-output*</code>, this can be used to close the
+<code>HTTP</code> connection to the client, and yet continue
+processing data.")
 {
   Stream *stream = get_stream_reference (vars);
 
