@@ -62,6 +62,18 @@ int symbol_small_prime = 107;
    symbol_set_default_package (). */
 Package *CurrentPackage = (Package *)NULL;
 
+/* An array of recently current packages.  The packages get on this list
+   with symbol_push_package (new_package), and are removed with
+   symbol_pop_package ().  Deleting a package from the global list side
+   effects this PDL -- possibly destroying the usefulness of the carat
+   referencing semantics. */
+Package **PackagePDL = (Package **)NULL;
+static int package_pdl_index = 0;
+static int package_pdl_size = 0;
+
+/* #define LowerCaseTranslationTable */
+
+#if !defined (LowerCaseTranslationTable)
 /* A translation table for ASCII characters.  This one has only uppercase
    characters in it. */
 static char u_stb[128] =
@@ -100,7 +112,10 @@ static char u_stb[128] =
   0x7c, /* | */    0x7d, /* } */    0x7e, /* ~ */    0x7f /*del*/
 };
 
-#if defined (LowerCaseTranslationTable)
+/* Symbol character translation table. */
+static char *stb = u_stb;
+
+#else /* defined (LowerCaseTranslationTable) */
 /* A translation table for ASCII characters.  This one has only lowercase
    characters in it. */
 static char l_stb[128] =
@@ -138,10 +153,11 @@ static char l_stb[128] =
   0x78, /* x */    0x79, /* y */    0x7a, /* z */    0x7b, /* { */
   0x7c, /* | */    0x7d, /* } */    0x7e, /* ~ */    0x7f /*  del*/
 };
-#endif /* LowerCaseTranslationTable */
 
 /* Symbol character translation table. */
-static char *stb = u_stb;
+static char *stb = l_stb;
+#endif /* LowerCaseTranslationTable */
+
 
 #define symchar(character) stb[(character & 0x7f)]
 
@@ -240,18 +256,87 @@ symbol_lookup_package (char *name)
 	{
 	  int name_len = strlen (name);
 
-	  for (i = 0; i < AP_index; i++)
-	    if ((AllPackages[i]->name_len == name_len) &&
-		(symbol_match_name ((Symbol *)AllPackages[i], name)))
-	      {
-		package = AllPackages[i];
-		break;
-	      }
+	  /* Special case magic package names.  If the name consists of
+	     all carats, then it is relative to the list of most recently
+	     accessed packages.  The number of carats signifies the offset
+	     from the current package, not in order of recently created
+	     packages, but in the order that the packages were most recently
+	     current. */
+	  for (i = 0; i < name_len; i++)
+	    if (name[i] != '^')
+	      break;
+
+	  /* If the package name was not a magic one, just look it up. */
+	  if (i != name_len)
+	    {
+	      for (i = 0; i < AP_index; i++)
+		if ((AllPackages[i]->name_len == name_len) &&
+		    (symbol_match_name ((Symbol *)AllPackages[i], name)))
+		  {
+		    package = AllPackages[i];
+		    break;
+		  }
+	    }
+	  else
+	    {
+	      /* Find the Nth most recently current package. */
+	      i = package_pdl_index - i;
+	      if (i > -1)
+		package = PackagePDL[i];
+	    }
 	}
     }
   return (package);
 }
 
+/* Push CurrentPackage onto the Package PDL, and make NEW_PACKAGE 
+   the current package. */
+void
+symbol_push_package (Package *new_package)
+{
+  if (package_pdl_index + 2 >= package_pdl_index)
+    PackagePDL = (Package **)xrealloc
+      (PackagePDL, package_pdl_size += (10 * sizeof (Package *)));
+
+  PackagePDL[package_pdl_index++] = CurrentPackage;
+  PackagePDL[package_pdl_index] = (Package *)NULL;
+  symbol_set_default_package (new_package);
+}
+
+/* Make the top of the Package PDL be the current package, and shrink
+   the PDL. */
+void
+symbol_pop_package (void)
+{
+  if (package_pdl_index)
+    {
+      Package *package = PackagePDL[--package_pdl_index];
+      PackagePDL[package_pdl_index] = (Package *)NULL;
+      symbol_set_default_package (package);
+    }
+}
+
+static void
+package_pdl_remove (Package *package)
+{
+  register int i;
+
+  if (package_pdl_index)
+    {
+      for (i = package_pdl_index - 1; i > -1; i--)
+	if (package == PackagePDL[i])
+	  {
+	    /* Move subsequent packages back. */
+	    while ((PackagePDL[i] = PackagePDL[i + 1]) != (Package *)NULL)
+	      i++;
+
+	    package_pdl_index--;
+
+	    break;
+	  }
+    }
+}
+
 /* Create a package named NAME suitable for storing symbols in.
    If the package already exists, return that package.
    You can create an anonymous package by passing NULL as the name. */
@@ -270,27 +355,23 @@ symbol_get_package (char *name)
       package->name_len = name ? strlen (name) : 0;
       package->table = symbol_create_table (0);
 
-      if (name != (char *)NULL)
-	{
-	  if ((AP_index + 2) > AP_slots)
-	    AllPackages = (Package **) xrealloc
-	      (AllPackages, (AP_slots += 10) * sizeof (Package *));
+      if ((AP_index + 2) > AP_slots)
+	AllPackages = (Package **) xrealloc
+	  (AllPackages, (AP_slots += 10) * sizeof (Package *));
 
-	  AllPackages[AP_index++] = package;
-	  AllPackages[AP_index] = (Package *)NULL;
+      AllPackages[AP_index++] = package;
+      AllPackages[AP_index] = (Package *)NULL;
 
-	  /* Fix up the name. */
-	  for (i = 0; i < package->name_len; i++)
-	    package->name[i] = symchar (package->name[i]);
-	}
+      /* Fix up the name. */
+      for (i = 0; i < package->name_len; i++)
+	package->name[i] = symchar (package->name[i]);
     }
 
   return (package);
 }
 
 /* Destroy the package PACKAGE, freeing all of the space that it
-   was using.  If the package was not anonymous, this deletes it
-   from the list of AllPackages. */
+   was using.  Delete the package from the list of AllPackages. */
 void
 symbol_destroy_package (Package *package)
 {
@@ -301,15 +382,13 @@ symbol_destroy_package (Package *package)
       if (package == CurrentPackage)
 	CurrentPackage = (Package *)NULL;
 
-      if ((package->name != (char *)NULL) && (AllPackages != (Package **)NULL))
+      if (AllPackages != (Package **)NULL)
 	{
-	  for (i = 0; i < AP_index; i++)
-	    if ((AllPackages[i]->name_len == package->name_len) &&
-		(symbol_match_name ((Symbol *)AllPackages[i], package->name)))
+	  for (i = AP_index; i > -1; i--)
+	    if (package == AllPackages[i])
 	      {
 		/* Move subsequent packages back. */
-		while ((AllPackages[i] = AllPackages[i + 1])
-		       != (Package *)NULL)
+		while ((AllPackages[i] = AllPackages[i+1]) != (Package *)NULL)
 		  i++;
 
 		AP_index--;
@@ -318,6 +397,9 @@ symbol_destroy_package (Package *package)
 	      }
 	}
 
+      /* Remove this package from the PackagePDL. */
+      package_pdl_remove (package);
+
       /* Now, simply free the package contents. */
       if (package->name) free (package->name);
       for (i = 0; i < package->table->rows; i++)
@@ -1045,7 +1127,7 @@ symbol_canonical_name (char *name)
 
 	  for (; ((name[i] != '\0') && (name[i] != ']')); i++);
 	  if (name[i] == ']')
-	    name[zero] = '\0';
+	    result[zero] = '\0';
 	}
     }
 
