@@ -104,13 +104,6 @@ int mhttpd_ssl_server = 0;
    MHTML::COOKIE-COMPATIBLE. */
 int mhtml_cookie_compatible = 0;
 
-static char *cookie_eating_clients[] =
-{
-  "Mozilla/", "OmniWeb", "ANT Fresco", "Emacs-W3/3", "GNNworks",
-  "Luxor", "NETCOMplete", "PRODIGY-WB/3", "Pueblo/2", "Spyglass Mosiac/3",
-  (char *)NULL };
-
-
 /* Return non-zero if this client is known to eat cookies. */
 static int
 client_eats_cookies (char *name)
@@ -120,14 +113,17 @@ client_eats_cookies (char *name)
   if (name != (char *)NULL)
     {
       register int i;
+      char **clients = symbol_get_values ("mhtml::cookie-eating-clients");
 
-      for (i = 0; cookie_eating_clients[i] != (char *)NULL; i++)
+      if (clients != (char **)NULL)
 	{
-	  if (strncmp (name, cookie_eating_clients[i],
-		       strlen (cookie_eating_clients[i])) == 0)
+	  for (i = 0; clients[i] != (char *)NULL; i++)
 	    {
-	      result = 1;
-	      break;
+	      if (strncmp (name, clients[i], strlen (clients[i])) == 0)
+		{
+		  result = 1;
+		  break;
+		}
 	    }
 	}
     }
