@@ -530,34 +530,49 @@ http_handle_request (HTTP_REQUEST *request, int fd)
 static PAGE *
 mhttpd_find_page_for_result (HTTP_RESULT *result)
 {
-  BPRINTF_BUFFER *path = bprintf_create_buffer ();
-  char *dir = pagefunc_get_variable ("mhtml::mhttpd-pages");
   PAGE *page = (PAGE *)NULL;
+  UserFunction *uf = mhtml_find_user_function ("mhttpd::default-document");
+  char *value = (char *)NULL;
+  char temp[20];
 
-  if (!dir)
-    dir = sv_DocumentRoot;
+  sprintf (temp, "%d", result->result_code);
+  pagefunc_set_variable ("mhttpd::result-code", temp);
+  pagefunc_set_variable ("mhttpd::requested-url",
+			 result->spec->logical_path ?
+			 result->spec->logical_path :
+			 result->spec->requested_path);
+  pagefunc_set_variable ("mhttpd::result-reason",
+			 mhttpd_result_reason (result->result_code));
+  pagefunc_set_variable ("mhttpd::request-method",
+			 result->request->method);
 
-  bprintf (path, "%s/Results/result-%d.mhtml", dir, result->result_code);
+  if (uf)
+    value = mhtml_evaluate_string ("<mhttpd::default-document>");
 
-  page = page_read_template (path->buffer);
-  free (path->buffer);
+  if (!empty_string_p (value))
+    {
+      page = page_create_page ();
+      page_set_contents (page, value);
+    }
+
+  if (value) free (value);
+
+  if (!page)
+    {
+      char *dir = pagefunc_get_variable ("mhtml::mhttpd-pages");
+      BPRINTF_BUFFER *path = bprintf_create_buffer ();
+
+      if (!dir)
+	dir = sv_DocumentRoot;
+
+      bprintf (path, "%s/Results/result-%d.mhtml", dir, result->result_code);
+
+      page = page_read_template (path->buffer);
+      free (path->buffer);
+    }
 
   if (page)
-    {
-      char temp[20];
-
-      sprintf (temp, "%d", result->result_code);
-      pagefunc_set_variable ("mhttpd::result-code", temp);
-      pagefunc_set_variable ("mhttpd::requested-url",
-			     result->spec->logical_path ?
-			     result->spec->logical_path :
-			     result->spec->requested_path);
-      pagefunc_set_variable ("mhttpd::result-reason",
-			     mhttpd_result_reason (result->result_code));
-      pagefunc_set_variable ("mhttpd::request-method", result->request->method);
-
-      page_process_page (page);
-    }
+    page_process_page (page);    
 
   return (page);
 }
@@ -670,6 +685,10 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
   char *answer = (char *)NULL;
   char *temp;
   char *permanent_p = pagefunc_get_variable ("mhtml::permanent-cookies");
+  char *no_cookies_p = pagefunc_get_variable ("mhtml::never-send-cookies");
+
+  if (!empty_string_p (no_cookies_p))
+    return ((char *)NULL);
 
   /* Set the SID cookie if that makes sense.
      I think it is important to pass the set-cookie command to each
@@ -1101,6 +1120,7 @@ mhttpd_build_environment (HTTP_RESULT *result)
 {
   MIME_HEADER **headers = result->request->headers;
   DOC_SPEC *spec = result->spec;
+  char *temp;
 
   /* Build an environment for this executable. */
   add_to_env ("PATH", pagefunc_get_variable ("mhtml::exec-path"));
@@ -1117,9 +1137,10 @@ mhttpd_build_environment (HTTP_RESULT *result)
   add_to_env ("REMOTE_ADDR", result->request->requester_addr);
   add_to_env ("REMOTE_USER", pagefunc_get_variable ("mhtml::remote-user"));
   add_to_env ("REMOTE_IDENT", pagefunc_get_variable ("mhtml::remote-ident"));
-  add_to_env
-    ("SERVER_PROTOCOL", mhtml_concat
-     (3, result->request->protocol, "/", result->request->protocol_version));
+  temp = mhtml_concat
+     (3, result->request->protocol, "/", result->request->protocol_version);
+  add_to_env ("SERVER_PROTOCOL", temp);
+  free (temp);
 
   add_to_env ("HTTP_AUTHORIZATION",
 	      mhttpd_get_mime_header (headers, "Authorization"));
@@ -1146,10 +1167,10 @@ mhttpd_build_environment (HTTP_RESULT *result)
 
   if (spec->content_length)
     {
-      char temp[40];
+      char lbuff[40];
 
-      sprintf (temp, "%d", spec->content_length);
-      add_to_env ("CONTENT_LENGTH", temp);
+      sprintf (lbuff, "%d", spec->content_length);
+      add_to_env ("CONTENT_LENGTH", lbuff);
     }
 
   add_to_env ("SCRIPT_NAME", spec->argv[0]);
@@ -1165,12 +1186,11 @@ mhttpd_build_environment (HTTP_RESULT *result)
   if (gbl_passed_sid)
     add_to_env ("SID", gbl_passed_sid);
 
-  /* Add SSL variables if present. */
+  /* Add additional HTTP variables if present. */
   if (environ != (char **)NULL)
     {
       register int i;
       char var_and_val[256];
-      char *temp;
 
       for (i = 0; environ[i] != (char *)NULL; i++)
 	{
@@ -1761,8 +1781,12 @@ decompose_and_post (char *enctype, char *data, int length)
 		  /* Just a regular variable.  Add this name/value to the
 		     POSTED package. */
 		  char *newname = (char *)xmalloc (20 + strlen (name));
+		  Symbol *sym;
+
 		  sprintf (newname, "POSTED::%s", name);
-		  pagefunc_set_variable (newname, value);
+		  sym = symbol_intern (newname);
+		  symbol_add_value (sym, value);
+		  free (newname);
 		}
 
 	      if (name) free (name);
@@ -1792,6 +1816,13 @@ mhttpd_metahtml_engine (HTTP_RESULT *result)
     {
       char *enctype;
 
+      {
+	PAGE *p = page_create_page ();
+	bprintf (p, "<*parser*::push-file %s>", result->spec->physical_path);
+	page_process_page (p);
+	page_free_page (p);
+      }
+
       enctype = mhttpd_get_mime_header
 	(result->request->headers, "Content-Type");
 
