@@ -29,6 +29,13 @@
    93101  */
 
 #include <signal.h>
-#if defined (sparc) || defined (__linux__) || defined (sgi) || defined (Solaris) || defined (__WINNT__) || defined (hpux)
-typedef void (*sig_t) (int);
+#if defined (__WINNT__)
+#  include <netdb.h>
+#  if !defined (HAVE_TYPE_SIG_T)
+#    define HAVE_TYPE_SIG_T 1
+#  endif
+#endif
+
+#if !defined (HAVE_TYPE_SIG_T)
+typedef RETSIGTYPE (*sig_t) (int);
 #endif
