@@ -117,7 +117,7 @@ mdb_insert_bp_1 (PAGE *page, int which, MDB_Breakpoint *bp, int position)
   /* The breakpoint surrounds the area to be broken.  We have to parse
      a single sexp forward. */
   insertion = bprintf_create_buffer ();
-  bprintf  (insertion, "<*mdb-break* %d ", which);
+  bprintf  (insertion, "<*MDB*::BREAK %d ", which);
 
   /* Now skip until we have read an entire expression. */
   expression_length = mdb_skip_sexp (page->buffer + position);
