@@ -782,11 +782,11 @@ mdb_info (MDBArgs)
 /* The honest to goodness workhorse function declaration. */
 static void pf_break_handler (PFunArgs);
 static void pf_include_handler (PFunArgs);
-static PFunDesc MDB_BreakPFunDesc = { "*MDB-BREAK*", 0, 0, pf_break_handler };
+static PFunDesc MDB_BreakPFunDesc = { "*MDB*::BREAK", 0, 0, pf_break_handler };
 static PFunDesc MDB_IncludePFunDesc = { "INCLUDE", 0, 0, pf_include_handler };
 extern char *get_positional_arg (Package *package, int position);
 
-/* The BREAK function looks like "<*mdb-break* which existing-statements>". */
+/* The BREAK function looks like "<*mdb*::break which existing-statements>". */
 static void
 pf_break_handler (PFunArgs)
 {
