@@ -20,6 +20,10 @@
    have a copy of the license, write to the Free Software Foundation,
    675 Mass Ave, Cambridge, MA 02139, USA. */
 
+#if defined (HAVE_CONFIG_H)
+#  include <config.h>
+#endif
+
 #if !defined (BUFSIZ)
 #include <stdio.h>
 #endif /* !BUFSIZ */
