@@ -197,7 +197,11 @@ rl_set_sighandler (int sig, SigHandler *handler)
 {
   struct sigaction act, oact;
 
+#if defined (_GNUC_)
   act.sa_handler = (typeof (act.sa_handler))handler;
+#else
+  act.sa_handler = handler;
+#endif
   act.sa_flags = 0;
   sigemptyset (&act.sa_mask);
   sigemptyset (&oact.sa_mask);
