@@ -1170,13 +1170,34 @@ loop_over_connections (void)
 		{
 #if defined (HAVE_GETPWNAM)
 		  char *user = pagefunc_get_variable ("mhtml::default-user");
+		  int group_set = 0;
+
+#if defined (HAVE_GETGRNAM)
+		  char *group = pagefunc_get_variable ("mhtml::default-group");
+
+		  if (!empty_string_p (group))
+		    {
+		      struct group *entry = getgrnam (group);
+
+		      if (entry != (group *)NULL)
+			{
+			  group_set++;
+			  setgid (group->gr_gid);
+			}
+		    }
+#endif /* HAVE_GETGRNAM */
 
 		  if (!empty_string_p (user))
 		    {
 		      struct passwd *entry = (struct passwd *)getpwnam (user);
 
 		      if (entry)
-			setuid (entry->pw_uid);
+			{
+			  if (!group_set)
+			    setgid (entry->pw_gid);
+
+			  setuid (entry->pw_uid);
+			}
 		    }
 #endif /* HAVE_GETPWNAM */
 		}
